Package: nara
Type: Package
Title: Native Raster Image Tools
Version: 1.0.2
Authors@R: c(
    person("Mike", "Cheng", role = c("aut", "cre", 'cph'), email = "mikefc@coolbutuseless.com"),
    person("Julien", "Vernay", role = "ctb", 
    comment = "Author of public domain code for thick polylines")
    )
Maintainer: Mike Cheng <mikefc@coolbutuseless.com>
Description: Native rasters are a core R image format which use a compact
    color representation. This color representation closely aligns with
    graphics device internals meaning that these images can be rendered quickly.
    This package provides functions to quickly create, manipulate and composite native
    rasters.
URL: https://github.com/coolbutuseless/nara,
        https://coolbutuseless.github.io/package/nara/index.html
BugReports: https://github.com/coolbutuseless/nara/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
LinkingTo: colorfast
Depends: R (>= 4.1.0)
Imports: colorfast
Suggests: testthat (>= 3.0.0), jpeg, fastpng, png, knitr, rmarkdown,
        magick
Config/testthat/edition: 3
Copyright: The included 'deer' sprites are by Calciumtrice and licensed
        under CC BY 3.0.  See 'inst/LICENSE-deer.txt' for more details.
        The included bitmap font 'spleen' was created by Frederic
        Cambus (c) 2018-2024. See 'inst/LICENSE-spleen.txt' for full
        LICENSE
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-28 10:34:36 UTC; mike
Author: Mike Cheng [aut, cre, cph],
  Julien Vernay [ctb] (Author of public domain code for thick polylines)
Repository: CRAN
Date/Publication: 2026-05-28 13:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-28 16:35:19 UTC; unix
Archs: nara.so.dSYM

Package: nara
Type: Package
Title: Native Raster Image Tools
Version: 1.0.2
Authors@R: c(
    person("Mike", "Cheng", role = c("aut", "cre", 'cph'), email = "mikefc@coolbutuseless.com"),
    person("Julien", "Vernay", role = "ctb", 
    comment = "Author of public domain code for thick polylines")
    )
Maintainer: Mike Cheng <mikefc@coolbutuseless.com>
Description: Native rasters are a core R image format which use a compact
    color representation. This color representation closely aligns with
    graphics device internals meaning that these images can be rendered quickly.
    This package provides functions to quickly create, manipulate and composite native
    rasters.
URL: https://github.com/coolbutuseless/nara,
        https://coolbutuseless.github.io/package/nara/index.html
BugReports: https://github.com/coolbutuseless/nara/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
LinkingTo: colorfast
Depends: R (>= 4.1.0)
Imports: colorfast
Suggests: testthat (>= 3.0.0), jpeg, fastpng, png, knitr, rmarkdown,
        magick
Config/testthat/edition: 3
Copyright: The included 'deer' sprites are by Calciumtrice and licensed
        under CC BY 3.0.  See 'inst/LICENSE-deer.txt' for more details.
        The included bitmap font 'spleen' was created by Frederic
        Cambus (c) 2018-2024. See 'inst/LICENSE-spleen.txt' for full
        LICENSE
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-28 10:34:36 UTC; mike
Author: Mike Cheng [aut, cre, cph],
  Julien Vernay [ctb] (Author of public domain code for thick polylines)
Repository: CRAN
Date/Publication: 2026-05-28 13:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-28 16:14:28 UTC; unix
Archs: nara.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,nativeRaster)
S3method(print,nativeRaster)
export(array_to_nr)
export(is_nativeraster)
export(magick_to_nr)
export(matrix_to_nr)
export(nr_blit)
export(nr_blit_multi)
export(nr_circle)
export(nr_color_replace)
export(nr_copy)
export(nr_copy_into)
export(nr_crop)
export(nr_crop2)
export(nr_desaturate)
export(nr_dither)
export(nr_fill)
export(nr_fliph)
export(nr_flipv)
export(nr_line)
export(nr_mask_begin)
export(nr_mask_end)
export(nr_new)
export(nr_new_from)
export(nr_point)
export(nr_polygon)
export(nr_polyline)
export(nr_rect)
export(nr_resize)
export(nr_rotate)
export(nr_scale)
export(nr_text_mono)
export(nr_threshold)
export(nr_to_array)
export(nr_to_magick)
export(nr_to_raster)
export(nr_transpose)
export(nr_tri_coords)
export(nr_tri_mesh)
export(nrs_to_gif)
export(nrs_to_mp4)
export(raster_to_nr)
import(colorfast)
import(grid)
importFrom(grDevices,dev.flush)
importFrom(grDevices,dev.hold)
useDynLib(nara, .registration=TRUE)

<h1 align="center">
<img src="https://raw.githubusercontent.com/subh-sk/nara/main/Logo/nara.png" width="300">
</h1><br>

<div align="center">
<p>
    <a href="https://pypi.org/project/Nara" target="_blank">
        <img src="https://img.shields.io/pypi/v/nara" />
    </a>
    <a href="https://pepy.tech/project/Nara" target="_blank">
        <img src="https://pepy.tech/badge/nara" />
    </a>
    <a href="https://github.com/subh-sk/Nara" target="_blank">
        <img src="https://img.shields.io/badge/nara-v0.4-green" />
    </a>
</p>
</div>

# nara: Your All-in-One AI and Utility Library

`nara` is a powerful Python library that provides a comprehensive suite of tools for AI-driven tasks, including real-time information retrieval, chatbot interactions, and code generation. It also offers a rich set of utility functions for tasks like temporary email creation, random data generation, caching, JSON manipulation, asynchronous task handling, and structured threading.  `nara` simplifies complex tasks, boosts productivity, and streamlines your workflow.
<!-- YATEESH DIVYANSH SUBHASH  -->
## Installation

Install `nara` using pip:
```bash
pip install nara
```

## Features

### AI-Powered Features

* **Code Generation:**
    * **`CreateFunc()`:** This decorator dynamically generates the implementation of a Python function based on a docstring or placeholder function definition. This feature leverages the power of large language models (LLMs) to automate code creation.
        ```python
        from nara import CreateFunc

        @CreateFunc()
        def calculate_area(length: int, width: int) -> int:
            '''Calculate the area of a rectangle.'''
            # ... (This function body will be replaced by generated code)
        ```
    * **`CreateTemplate()`:**  Generates entire Python templates (e.g., Flask apps, Machine Learning models) from high-level descriptions. This feature is ideal for quickly scaffolding projects and generating boilerplate code.
        ```python
        from nara import CreateTemplate

        CreateTemplate("Create a Flask template with a route that returns 'Hello, World!'") 
        ```

* **Large Language Model (LLM) Interaction:**
    * **`nara.llm`:** This module provides a unified interface for interacting with various LLMs, including Groq, OpenAI, Cohere, Sambanova, Together, and Ollama.  You can easily switch between different models and customize parameters like temperature and max tokens. Supports both synchronous and streaming responses.
        ```python
        from nara.llm._groq import Groq, LLAMA_32_90B_TEXT_PREVIEW

        llm = Groq(LLAMA_32_90B_TEXT_PREVIEW)  # Initialize the LLM
        response = llm.run("Translate 'Hello, World!' to Spanish.")
        print(response) # Output: Hola, Mundo!
        ```
        For more advanced use, see the streaming example in the Usage section.

* **Prompt Engineering:**
    * **`nara.extra.prompt`:** Provides tools for constructing and managing prompts for LLMs, including support for different roles (system, user, assistant), text, images, and function calls.
        ```python
        from nara.extra.prompt import Prompt, Role, Text

        prompt = Prompt(role=Role.user, template=[Text("What is the capital of France?")])
        print(prompt.prompt) # Output: What is the capital of France?
        ```


### Utility Features

* **Temporary Email:**
    * **`mailOtp()`, `onlyMail()`, `mailUrl()`:** Create temporary email addresses for receiving OTPs or URLs, useful for testing and automation. (Powered by `mailtm` and `temp-mail.io`).

        ```python
        from nara.extra.temp_mail import mailOtp

        email_generator = mailOtp(OtpLength=6, Printable=True, Timeout=60)
        email_address = next(email_generator)  # Get the temporary email address
        print(f"Your temporary email: {email_address}")

        otp = next(email_generator) # Wait for the OTP
        print(f"Received OTP: {otp}")  
        ```

* **Random Data Generation:**
    * **`RandomDataGenerator`:** Easily generate realistic fake data for various purposes, such as testing, populating databases, and creating mock data. Includes names, addresses, usernames, passwords, and more. (Powered by `faker`).

        ```python
        from nara.extra.fake import RandomDataGenerator

        generator = RandomDataGenerator()
        name = generator.name()
        address = generator.address()
        print(f"Name: {name}, Address: {address}")
        ```
* **Caching:**
    * **`CacheManager`:** Simplifies caching function results to disk, improving performance by avoiding redundant computations.

        ```python
        from nara.extra.extended_json import CacheManager

        cache_manager = CacheManager("my_cache.json")

        @cache_manager.cache
        def expensive_function(arg1, arg2):
            # ... complex and time-consuming computation ...
            return result

        # Subsequent calls with the same arguments will retrieve the cached result
        result = expensive_function(1, 2)  
        ```

* **JSON Manipulation:**
    * **`loadJson(FileName="Json.json", PickRandom=None)`:** Loads JSON data from a file.  If `PickRandom` is an integer, it returns a dictionary or list containing that many randomly selected items from the JSON data.
        ```python
        from nara.extra.extended_json import loadJson

        data = loadJson("data.json")  # Load all data
        random_data = loadJson("data.json", PickRandom=2)  # Load 2 random items
        ```
        **Error Handling:** Raises `FileNotFoundError` if the file doesn't exist and `ValueError` for invalid JSON or if `PickRandom` exceeds the data length.

    * **`jsonList(FileName="JsonList.json", **kwargs)`:**  Appends key-value pairs (provided as keyword arguments) as a new dictionary entry to a JSON list. Creates a new list if the file doesn't exist or handles invalid data.
        ```python
        from nara.extra.extended_json import jsonList

        jsonList(FileName="my_list.json", name="Alice", age=30)
        jsonList(FileName="my_list.json", name="Bob", age=25)
        ```
        **Error Handling:** Displays warnings for non-existent files, empty files, or files not containing a list. Raises `ValueError` if the JSON file is empty or does not contain a list.

    * **`jsonDict(Key, Value, FileName="JsonDict.json")`:** Updates or adds a single `Key`-`Value` pair to a JSON dictionary stored in a file. Creates a new dictionary if the file doesn't exist.
        ```python
        from nara.extra.extended_json import jsonDict

        jsonDict("name", "Charlie", FileName="my_dict.json")
        jsonDict("age", 35, FileName="my_dict.json")
        ```
        **Error Handling:**  Displays warnings for similar file errors as `jsonList`.  Raises `ValueError` if the JSON file is empty or does not contain a dictionary.


* **Asynchronous Task Handling:**
    * **`asyncThreadedTask(thread_count=1, *threading_args, **threading_kwargs)`:** Executes an asynchronous function in a separate thread. Useful for running async operations within synchronous code or managing blocking I/O. This example demonstrates the usage with and without parameters:
    ```python
    import asyncio
    from nara import asyncThreadedTask
    
    @asyncThreadedTask
    async def async_function():
        print("Async function started")
        await asyncio.sleep(2)  # Simulate some async operation
        print("Async function finished")

    @asyncThreadedTask(2) # Use two threads
    async def async_function_2():
        print("Async function 2 started")
        await asyncio.sleep(2)
        print("Async function 2 finished")


    async def main():
        await async_function()
        await async_function_2()


    asyncio.run(main())
    ```


    * **`runMultipleTimes(count=1)`:** Runs an asynchronous function multiple times concurrently using `asyncio.gather`.
        ```python
        import asyncio
        from nara.extra.tools import runMultipleTimes
    
        @runMultipleTimes(5)  # Run the function 5 times concurrently
        async def my_async_task():
            print("Task started")
            await asyncio.sleep(1)
            print("Task finished")
    
        asyncio.run(my_async_task())
        ```


* **Structured Threading:**
    * **`retry(retries=3, delay=1)`:**  This decorator retries a function call if it raises an exception. It will retry up to `retries` times, pausing for `delay` seconds between attempts.
        ```python
        from nara.extra.tools import retry
        import random

        @retry(retries=5, delay=2)
        def might_fail():
            if random.random() < 0.5:
                raise Exception("Function failed!")
            print("Function succeeded!")
        
        might_fail()
        ```
    * **`repeatForever(sleep_interval=None, end_after=None)`:**  Executes a function or coroutine repeatedly.  `sleep_interval` specifies the pause between calls. `end_after` (a human-readable time string, e.g., "10 minutes") allows setting a time limit.
        ```python
        from nara.extra.tools import repeatForever
        import asyncio
        import time

        @repeatForever(sleep_interval=1, end_after="5 seconds")  # Repeat every second for 5 seconds
        def print_hello():
            print("Hello")

        @repeatForever(sleep_interval=1, end_after="5 seconds")
        async def async_print_hello():
            print("Async Hello")
            await asyncio.sleep(0.5) # Simulate some work
        

        print_hello() # Normal function example

        async def main(): # Asyncio function example
            await async_print_hello()

        asyncio.run(main())

        ```
        **Error Handling:** `repeatForever` raises `humanfriendly.InvalidTimespan` if `end_after` is not a valid timespan string.


* **File Management:**
    * **`clearPycacheDirectories(directory, display_status=False)`:** Recursively deletes all `__pycache__` directories within the given `directory`. Set `display_status=True` to print which directories were removed.
        ```python
        from nara.extra.file_manager import clearPycacheDirectories

        clearPycacheDirectories(".", display_status=True)  # Clear __pycache__ in the current directory and subdirectories
        ```

    * **`displayFileStructure(directory=os.getcwd(), tree=None, console=None)`:**  Prints a tree-like representation of the directory structure to the console.
        ```python
        from nara.extra.file_manager import displayFileStructure
        
        displayFileStructure(".")  # Display the file structure of the current directory
        ```

* **Date and Time Utilities:**
    * **`CurrentDateTime`:**  Retrieve current date and time information for various countries using external API.  Uses [this API](https://time-epbx.onrender.com/).
        ```python
        from nara.extra.date_time import CurrentDateTime

        dt = CurrentDateTime()
        country_codes = dt.countryCodes()  # Get available country codes
        indian_time = dt.time("IN", encode=True)  # Get current Indian time as datetime object
        us_date = dt.date("US") # Get current US date as string
        print(f"Country Codes: {country_codes}")
        print(f"Indian Time: {indian_time}")
        print(f"US Date: {us_date}")


        ```


* **Testing Tools:**
    * **`saveTestResults(OutputFile, **outerkwargs)`:**  This decorator saves test results to a JSON file. `outerkwargs` can include functions to calculate additional metrics (like mean or max) based on the test result.
        ```python
        from nara.extra.extended_json.testing_tools import saveTestResults

        @saveTestResults("results.json", mean=lambda x: sum(x)/len(x), maximum=max)
        def test_function(a, b):
            return [a + b, a * b]

        test_function(2, 3)  # Save results and calculated metrics to results.json
        ```

    * **`LoadTestResults(inputFile)`:**  Loads and returns the test results saved by `saveTestResults`.
        ```python
        from nara.extra.extended_json.testing_tools import LoadTestResults

        results = LoadTestResults("results.json")
        print(results)
        ```

* **SQL Utilities:** (Examples use a database file named `mydatabase.db`)
    * **`SqlQueue(db_file)`:**  Implements a queue using a SQLite database, enabling persistent queueing even if your application restarts.
        ```python
        from nara.extra.extended_sql import SqlQueue
        
        q = SqlQueue("mydatabase.db")
        q.put(10)
        q.put("Hello")
        
        retrieved_item = q.get() # Returns 10 (FIFO)
        print(retrieved_item)
        ```

    * **`TextStore(dbName)`:** Stores and manages text lines in a SQLite database, providing methods for adding, updating, deleting, and retrieving text lines by ID or slices. 
        ```python
        from nara.extra.extended_sql import TextStore
        
        ts = TextStore("mydatabase.db")
        ts.addRecord("This is a test line.")
        ts.addRecord("Another line of text.")
        
        all_records = ts.listRecords() # Retrieve all records
        print(all_records)
        ```

    * **`SqlList(DbName, TableName)`:** Enables you to use a SQLite table as if it were a Python list. You can append, access by index, slice, extend, pop, remove, and perform other list-like operations. Elements are stored as JSON strings.

        ```python
        from nara.extra.extended_sql import SqlList

        sl = SqlList("mydatabase.db", "my_list_table")
        sl.append({"name": "Alice", "age": 30})
        sl.append({"name": "Bob", "age": 25})
        
        print(sl[0])  # Access by index (returns a dictionary)
        print(len(sl)) # Get the length of the SqlList
        ```

    * **`SQLiteDict(db_name)`:** Uses a SQLite database as a persistent dictionary. Supports dictionary operations like setting, getting, deleting items, checking membership (`in`), and iterating through keys or items. Stores all data types as strings.
        ```python
        from nara.extra.extended_sql import SQLiteDict

        sd = SQLiteDict("mydatabase.db")
        sd['name'] = "Charlie"
        sd['age'] = 35
        
        print(sd['name'])  # Access by key
        print("name" in sd) # Check for key existence
        ```
        **Error Handling:** The `get` method will return `None` if the requested key is not found and the data in the "value" column cannot be evaluated as valid Python code, the SQLiteDict's `get` method will return `None`, instead of raising an exception. It will also print an error message indicating the failure to evaluate the value.


* **Time Utilities:**
    * **`timeIt(func)`:** A decorator to measure and print the execution time of a function (both synchronous and asynchronous).

        ```python
        import asyncio
        from nara.extra.tools import timeIt
        import time

        @timeIt
        def my_function():
            time.sleep(1) # Simulate a second of work

        @timeIt
        async def my_async_function():
            await asyncio.sleep(1)

        my_function()
        asyncio.run(my_async_function())

        ```




## Usage

###  LLM Streaming Example (Groq)

```python
from nara.llm._groq import Groq, LLAMA_32_90B_TEXT_PREVIEW

llm = Groq(LLAMA_32_90B_TEXT_PREVIEW)  # Initialize the LLM

for chunk in llm.streamRun("Write a short story about a cat who goes on an adventure."):
    print(chunk, end="", flush=True) # Print the story as it's generated
```

###  Prompt with Function Example

```python
from nara.extra.prompt import Prompt, Role, Text, Function

def get_current_time():
    from datetime import datetime
    return datetime.now().strftime("%Y-%m-%d %H:%M:%S")


prompt = Prompt(role=Role.user, template=[
    Text("The current time is: "),
    Function(get_current_time), 
    Text(".  Tell me a fun fact about cats.")
])

print(prompt.prompt)
```
<div align="center">

# ✨ **Authors** ✨

![Author](https://img.shields.io/badge/Divyansh_Shukla-purple?style=for-the-badge&logo=github&logoColor=white)
![Author](https://img.shields.io/badge/Yateesh_Reddy-green?style=for-the-badge&logo=github&logoColor=white)
![Author](https://img.shields.io/badge/Subhash_Kumar-blue?style=for-the-badge&logo=github&logoColor=white)


</div>

## Contributing

We welcome contributions! See [CONTRIBUTING.md](CONTRIBUTING.md) for guidelines.

## License

MIT License. See [LICENSE.txt](LICENSE.txt) for details.


## Contact

naravirtualai@gmail.com



## Acknowledgements

We thank all contributors and the open-source community for their support.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, echo = FALSE------------------------------------------------------
library(nara)

## ----eval=FALSE---------------------------------------------------------------
# library(nara)
# w <- 320
# h <- 200
# 
# set.seed(1)
# N <- 20
# xs   <- runif(N, 0, w)
# ys   <- runif(N, 0, h)
# rs   <- runif(N, 2, 6)
# vx   <- rnorm(N, sd = 3)
# cols <- sample(heat.colors(N))
# 
# # Storage for animation frames
# anim <- list()
# 
# for (i in seq(30)) {
#   # create image
#   nr <- nr_new(w, h, fill = 'grey90')
# 
#   # update the x position and wraparound
#   xs <- xs + vx
#   xs <- ifelse(xs < 0, xs + w, xs)
#   xs <- ifelse(xs > w, xs - w, xs)
# 
#   # draw circles
#   nr_circle(nr, xs, ys, rs, cols)
# 
#   # store animatino frame
#   anim[[i]] <- nr
# }
# 
# nrs_to_gif(anim, "working/demo.gif")
# # nrs_to_mp4(anim, "working/demo.mp4")

## ----eval=FALSE---------------------------------------------------------------
# x11(..., type = 'dbcairo', antialias = 'none')
# dev.control(displaylist = 'inhibit')
# 
# for (i in seq(nframes)) {
#   nr <- generate_image_somehow()
#   dev.hold()
#   plot(nr)
#   dev.flush()
#   Sys.sleep(0.03) # Sleep maybe?
# }

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Start/end region of masked drawing
#' 
#' These functions are used to globally set a mask which determines where
#' pixels are affected by drawing functions.
#' 
#' Drawing operations between
#' \code{nr_mask_begin()} and \code{nr_mask_end()} calls will only affect pixels
#' where the \code{mask} is not transparent.
#' 
#' @inheritParams nr_fill
#' @param mask native raster image to use as mask. Must be the same size as
#'        \code{nr}.
#'        
#' @return Invisibly return the original native raster image which has been 
#'         modified in-place
#'         
#' @examplesIf interactive()
#' mask <- deer[[1]] |> nr_resize(600, 400)
#' plot(mask)
#' nr <- nr_new_from(mask)
#' 
#' nr_mask_begin(nr, mask = mask)
#' nr_circle(nr, 300, 200, 200, fill = 'blue', color = 'black')
#' nr_circle(nr, 350, 200, 100, fill = 'darkgreen', color = 'black')
#' nr_mask_end(nr)
#' 
#' plot(nr)
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_mask_begin <- function(nr, mask) {
  invisible(
    .Call(nr_mask_begin_, nr, mask)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_mask_begin
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_mask_end <- function(nr) {
  invisible(
    .Call(nr_mask_end_, nr)
  )
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Convert a 'magick' image to native raster image
#' 
#' @inheritParams matrix_to_nr
#' @inheritParams array_to_nr
#' @param im image from the \code{magick} package
#' 
#' @return New native raster image
#' 
#' @examplesIf interactive() && requireNamespace('magick', quietly = TRUE)
#' im <- magick::logo
#' nr <- magick_to_nr(im)
#' plot(nr)
#' @family conversion functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
magick_to_nr <- function(im, dst = NULL) {
  if (requireNamespace('magick', quietly = TRUE)) {
    im <- magick::image_convert(im, format = 'rgba', matte = TRUE)
    if (length(im) == 1) {
      .Call(magick_to_nr_, magick::image_data(im), dst)
    } else {
      lapply(seq_along(im), function(i) {
        .Call(magick_to_nr_, magick::image_data(im, frame = i), dst)
      })
    }
  } else {
    stop("magick_to_nr() requires the {magick} package")
  }
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname magick_to_nr
#' @family conversion functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_to_magick <- function(nr) {
  if (requireNamespace('magick', quietly = TRUE)) {
    magick::image_read(nr)
  } else {
    stop("nr_to_magick() requires the {magick} package")
  }
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Convert a list of native rasters to an animated gif
#' 
#' @param nr_list list of native raster image
#' @param filename filename of gif
#' @param verbose logical. default FALSE
#' @param framerate frames per second
#' @param ... other arguments passed to \code{magick::image_write_gif()}
#' @return None.
#' @examplesIf interactive() && requireNamespace('magick', quietly = TRUE)
#' im <- magick::logo
#' nr1 <- magick_to_nr(im)
#' nr2 <- nr_copy(nr1)
#' nrs_to_gif(list(nr1, nr2), "nothing.gif")
#' @family conversion functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nrs_to_gif <- function(nr_list, filename, verbose = FALSE, framerate = 30, ...) {
  if (requireNamespace('magick', quietly = TRUE)) {
    nr_list <- lapply(nr_list, nr_to_magick)
    ims <- do.call(magick::image_join, nr_list)
    magick::image_write_gif(ims, path = filename, progress = verbose, delay = 1/framerate, ...)
  } else {
    stop("nrs_to_gif() requires the {magick} package")
  }
  invisible(nr_list)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Convert a list of native rasters to an mp4 video file
#' 
#' @param nr_list list of native raster image
#' @param filename mp4 filename
#' @param verbose logical. default FALSE
#' @param ... other arguments passed to \code{magick::image_write_video()}
#' @return None.
#' @examplesIf interactive() && requireNamespace('magick', quietly = TRUE)
#' im <- magick::logo
#' nr1 <- magick_to_nr(im)
#' nr2 <- nr_copy(nr1)
#' nrs_to_mp4(list(nr1, nr2), "nothing.mp4")
#' @family conversion functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nrs_to_mp4 <- function(nr_list, filename, verbose = FALSE, ...) {
  if (requireNamespace('magick', quietly = TRUE)) {
    nr_list <- lapply(nr_list, nr_to_magick)
    ims <- do.call(magick::image_join, nr_list)
    magick::image_write_video(ims, path = filename, verbose = verbose, ...)
  } else {
    stop("nrs_to_mp4() requires the {magick} package")
  }
  invisible(nr_list)
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Convert native raster images to/from other R objects
#'
#' @inheritParams nr_rect
#' @param ras standard R raster i.e. a character matrix of hex color values
#' @param arr 3d numeric array representing R,G,B,A values with dimensions [nrow, ncol, 4] or
#'        [nrow, ncol, 3]. Each value is in range [0,1].
#' @param dst destination native raster image. If NULL (the default) a new 
#/        native raster image will be created.
#'
#' @return raster, array or native raster image
#' 
#' @examples
#' nr <- nr_new(12, 8, 'hotpink')
#' nr_to_raster(nr)
#' 
#' nr_to_array(nr)
#' @family conversion functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_to_raster <- function(nr) {
  .Call(nr_to_raster_, nr)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_to_raster
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
raster_to_nr <- function(ras, dst = NULL) {
  .Call(raster_to_nr_, ras, dst)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_to_raster
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_to_array <- function(nr) {
  .Call(nr_to_array_, nr)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_to_raster
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
array_to_nr <- function(arr, dst = NULL) {
  .Call(array_to_nr_, arr, dst)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Convert a numeric matrix to native raster image
#' 
#' @param mat integer matrix
#' @param palette vector of colors.  For an integer matrix, this palette must 
#'        contain at least as many colors as the maximum integer value in \code{mat}. 
#'        For a numeric matrix, any length palette is allowed, and the nearest
#'        corresponding color is chosen (no interpolation is done).
#' @param dst destination native raster image. If NULL (the default) a 
#'        new native raster image will be created  If a native raster image 
#'        is supplied here, it must have the exact dimensions to match the matrix        
#' @param fill Color to be used for values < 1 when input is an integer matrix.  
#'        Default: 'transparent'.
#' @param min,max assumed range for the numeric data.  values from the palette
#'        will be interpolated using this range as the extents.  An error
#'        will occur if a value lies outside this range. Default: (0, 1)
#' @return native raster image
#'
#' @examples
#' # integer matrix
#' m <- matrix(1:12, 3, 4)
#' m
#' palette <- rainbow(12)
#' nr <- matrix_to_nr(m, palette) 
#' plot(nr)
#' 
#' 
#' # numeric matrix
#' m <- matrix(runif(20 * 30), 20, 30)
#' palette <- c('red', 'blue', 'black', 'green', 'blue', 'yellow')
#' nr <- matrix_to_nr(m, palette)
#' plot(nr)
#' @family conversion functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
matrix_to_nr <- function(mat, palette, fill = 'transparent', min = 0.0, max = 1.0, dst = NULL) {
  .Call(matrix_to_nr_, mat, palette, fill, min, max, dst)
}

test_that("integer matrix to nr works", {
  mat <- matrix(1:24, 8, 3)
  
  nr <- matrix_to_nr(mat, palette = rainbow(24))
  
  expected <-matrix(
    colorfast::col_to_int(rainbow(24))
    , 8, 3, byrow = FALSE
  ) |> t()
  dim(expected) <- c(8, 3) 
  
  expect_equal(nr, expected, ignore_attr = TRUE)
})




test_that("numeric matrix to nr works", {
  mat <- matrix(seq(0, 1, length = 24), 8, 3)
  
  nr <- matrix_to_nr(mat, palette = rainbow(24))
  # plot(nr)
  
  colorfast::col_to_int(rainbow(24))
  
  
  expected <-matrix(
    colorfast::col_to_int(rainbow(24))
    , 8, 3, byrow = FALSE
  ) |> t()
  dim(expected) <- c(8, 3) 
  
  expect_equal(nr, expected, ignore_attr = TRUE)
})

set.seed(2022)


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Round trip to raster (with alpha)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
test_that("round trip from array works (with alpha)", {
  
  im <- testthat::test_path("image/Rlogo-tiny.png")
  arr <- fastpng::read_png(im, type = 'array')
  # plot(as.raster(arr), interpolate = FALSE)

  nr <- array_to_nr(arr)
  # plot(nr)
  
  arr2 <- nr_to_array(nr)
  # plot(as.raster(arr2), interpolate = FALSE)

  attr(arr2, "class") <- NULL
  
  expect_identical(arr, arr2)
})


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Round trip to raster (without alpha)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
test_that("round trip from array works (without alpha)", {
  
  im <- testthat::test_path("image/Rlogo-tiny.png")
  arr <- fastpng::read_png(im, type = 'array')
  arr <- arr[,,1:3]
  # plot(as.raster(arr), interpolate = FALSE)
  
  nr <- array_to_nr(arr)
  # plot(nr)
  
  arr2 <- nr_to_array(nr)
  # plot(as.raster(arr2), interpolate = FALSE)
  
  attr(arr2, "class") <- NULL
  
  expect_identical(arr, arr2[,,1:3])
})

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Check if object is a native raster image
#' 
#' @param x object to check
#' 
#' @return logical. TRUE if object is a native raster image
#' 
#' @examples
#' is_nativeraster(mtcars)
#' 
#' nr <- nr_new(100, 100)
#' is_nativeraster(nr)
#' 
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
is_nativeraster <- function(x) {
  inherits(x, 'nativeRaster')
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Create a native raster image
#'
#' A native raster image in R looks like an integer matrix, but is interpreted
#' differently by graphics devices:
#'
#' \itemize{
#'   \item{The data should be treated as RGBA pixels in row-major ordering}
#'   \item{Each 32-bit integer should be interpreted as 4-bytes - one for each
#'         of the R, G, B and A color channels}
#' }
#'
#' @param width,height Image dimensions in pixels
#' @param nr native raster image to use as the template for the size of the 
#'        new image in \code{nr_new_from()}
#' @param fill Background fill color as a character string. Either a standard R color
#'        (e.g. 'blue', 'white')
#'        or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
#'        or \code{#rgb}
#'
#' @return native raster image
#' 
#' @examples
#' nr <- nr_new(400, 300, 'hotpink')
#' plot(nr)
#' 
#' @import colorfast
#' @family image creation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_new <- function(width, height, fill = 'white') {
  nr <- .Call(nr_new_, width, height)
  .Call(nr_fill_, nr, fill)
  nr
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_new
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_new_from <- function(nr, fill = 'white') {
  nr_new(width = ncol(nr), height = nrow(nr), fill = fill)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Create a new native raster image and copy the dimensions and
#' contents from an existing image
#'
#' @inheritParams nr_fill
#' 
#' @return New native raster image
#' 
#' @examples
#' nr1 <- nr_new(200, 200, 'hotpink')
#' nr2 <- nr_copy(nr1)
#' plot(nr2)
#' @family image creation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_copy <- function(nr) {
  .Call(nr_copy_, nr)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Copy the contents of one native raster image into another.
#'
#' The source and destination native raster images must have the same dimensions.
#'
#' If the native raster images are of different sizes or alpha blending is
#' required, use the \code{\link{nr_blit}()} function.
#'
#' @param src,dst Source and destination native raster images
#'
#' @return Invisibly return the supplied 'dst' native raster image which was been
#'         modified in-place
#' 
#' @examples
#' nr1 <- nr_new(200, 100, 'hotpink')
#' nr2 <- nr_new(200, 100, 'green')
#' nr_copy_into(nr1, nr2)
#' plot(nr1)
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_copy_into <- function(dst, src) {
  invisible(.Call(nr_copy_into_, dst, src))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Fill a native raster image with the given color
#'
#' @param nr native raster image
#' @param color Color as a character string. Either a standard R color
#'        (e.g. 'blue', 'white')
#'        or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
#'        or \code{#rgb}
#'        
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place       
#' 
#' @examples
#' nr <- nr_new(400, 300, 'hotpink')
#' nr_fill(nr, 'blue')
#' plot(nr)
#' 
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_fill <- function(nr, color) {
  invisible(.Call(nr_fill_, nr, color))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Crop a section out of a native raster image into a new image
#' 
#' @inheritParams nr_fill
#' @param x,y,w,h dimensions of cropped section
#' @param loc dimensions of cropped section. A vector of 4 values 
#'        i.e. \code{c(x, y, w, h)}
#' 
#' @return New native raster image
#' 
#' @examples
#' nr <- deer[[1]]
#' dim(nr)
#' plot(nr)
#' 
#' nr2 <- nr_crop(nr, 16, 0, 16, 16)
#' dim(nr2)
#' plot(nr2)
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_crop <- function(nr, x, y, w, h) {
  dst <- nr_new(w, h)
  nr_blit(dst = dst, src = nr, 
          x = 0, y = 0, 
          xsrc = x, ysrc = y, 
          w = w, h = h, 
          hjust = 0, vjust = 0,
          use_alpha = FALSE)
  dst
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_crop
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_crop2 <- function(nr, loc) {
  nr_crop(nr, loc[[1]], loc[[2]], loc[[3]], loc[[4]])
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Flip a native raster image vertically
#' 
#' @inheritParams nr_fill
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' 
#' @examples
#' nr <- deer[[1]] |> nr_copy()
#' plot(nr)
#' nr_flipv(nr)
#' plot(nr)
#' @family transformation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_flipv <- function(nr) {
  invisible(.Call(nr_flipv_, nr))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Flip a native raster image horizontally
#' 
#' @inheritParams nr_fill
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' 
#' @examples
#' nr <- nr_new(400, 200, 'white')
#' nr_rect(nr, 0, 0, 30, 15)
#' plot(nr)
#' nr_fliph(nr)
#' plot(nr)
#' @family transformation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_fliph <- function(nr) {
  invisible(.Call(nr_fliph_, nr))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Rotate a native raster image by 90,180,270 degrees
#' @inheritParams nr_fill
#' @param angle one of 0,90,180,270
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' 
#' @examples
#' nr <- nr_new(20, 10, 'hotpink')
#' dim(nr)
#' nr_rotate(nr, 90)
#' dim(nr)
#' @family transformation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_rotate <- function(nr, angle) {
  invisible(
    .Call(nr_rotate_, nr, angle)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Transpose
#' @inheritParams nr_fill
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' @examples
#' nr <- nr_new(20, 10, 'hotpink')
#' dim(nr)
#' nr_transpose(nr)
#' dim(nr)
#' @family transformation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_transpose <- function(nr) {
  invisible(
    .Call(nr_transpose_, nr)
  )  
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Print method
#' 
#' @param x native raster image
#' @param ... ignored
#' @return None
#' @examples
#' nr <- nr_new(500, 400)
#' print(nr)
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
print.nativeRaster <- function(x, ...) {
  d <- dim(x)
  cat(sprintf("<nativeRaster> object: rows:%i  cols:%i\n", d[1], d[2]))
  invisible(x)
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Plot a native raster image (after first clearning the device)
#' 
#' @param x native raster image
#' @param y ignored
#' @param ... other arguments passed to \code{grid::grid.raster()}
#'
#' @return Invisibly return the supplied native raster image
#' 
#' @examples
#' nr <- nr_new(200, 100, 'hotpink')
#' plot(nr)
#' 
#' @import grid
#' @importFrom grDevices dev.flush dev.hold
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
plot.nativeRaster <- function(x, y, ...) {
  grid::grid.newpage()
  grDevices::dev.hold()
  grid::grid.raster(x, interpolate = FALSE, ...)
  grDevices::dev.flush()
  invisible(x)
}

test_that("Crop work works", {

  nr <- nr_new(400, 400, 'hotpink')
  nr2 <- nr_crop(nr, 0, 0, 4, 3)
  expect_equal(ncol(nr2), 4)
  expect_equal(nrow(nr2), 3)
  
  expect_identical(unique(nr2), colorfast::col_to_int('hotpink'))
})

test_that("NN scale works", {
  
  black <- colorfast::col_to_int('black')
  clear <- colorfast::col_to_int('#00000000')
  half  <- colorfast::col_to_int('#0000007f')
  
  nr <- nr_new(2, 2, fill = clear)
  nr_point(nr, 0, 0, color = black)
  nr_point(nr, 1, 1, color = black)

  nr3 <- nr_scale(nr, 3, algo = 'nn')  
  expect_equal(nrow(nr3), 6)  
  expect_equal(ncol(nr3), 6)  
  
  expect_equal(
    nr3, 
    matrix(c(
      black, black, black, clear, clear, clear,
      black, black, black, clear, clear, clear,
      black, black, black, clear, clear, clear,
      clear, clear, clear, black, black, black,
      clear, clear, clear, black, black, black,
      clear, clear, clear, black, black, black),
      nrow = 6, ncol = 6, byrow = TRUE
    ), ignore_attr = TRUE
  )
  
  
  
  nr3 <- nr_scale(nr, 1.5, algo = 'bilinear')  
  expect_equal(nrow(nr3), 3)  
  expect_equal(ncol(nr3), 3)  
  
  expect_equal(
    nr3,
    matrix(c(
      black, half, clear,
      half, half, half,
      clear, half, black
    ), nrow = 3, ncol = 3, byrow = TRUE), ignore_attr = TRUE
  )
    
})

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Resize a native raster by specifying the output dimensions
#' 
#' @inheritParams nr_fill
#' @param algo 'nn' for nearest neighbor (the default), or 'bilinear' for 
#'        bilinear interpolation.
#' @param width,height dimensions for output image
#' @return New native raster image
#' @examples
#' stretched <- deer[[1]] |> 
#'     nr_copy() |> 
#'     nr_resize(100, 40, algo = 'nn')
#' plot(stretched)
#' @family resizing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_resize <- function(nr, width, height, algo = 'nn') {
  if (algo == 'nn') {
    .Call(nr_resize_nn_, nr, width, height)
  } else {
    .Call(nr_resize_bilinear_, nr, width, height)
  }
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Resize a native raster image using a scale factor
#' 
#' @inheritParams nr_resize
#' @param scale scale factor
#' 
#' @return New native raster image
#' @examples
#' big <- deer[[1]] |> nr_scale(2)
#' plot(big)
#' @family resizing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_scale <- function(nr, scale, algo = 'nn') {
  nr_resize(nr, width = scale * ncol(nr), height = scale * nrow(nr), algo = algo)
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw points on a native raster image
#'
#' @inheritParams nr_fill
#' @param x,y Vectors of point coordinates
#' @param color Vector of colors
#' @param use_alpha Use alpha channel when drawing? Logical. Default: TRUE 
#'
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'
#' @examples
#' w <- 200
#' h <- 150
#' nr <- nr_new(w, h, 'grey80')
#' 
#' coords <- expand.grid(x = seq(w) - 1, y = seq(h) - 1)
#' cols   <- sample(rainbow(nrow(coords)))
#' nr_point(nr, x = coords$x, y = coords$y, color = cols) 
#' plot(nr)
#' 
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_point <- function(nr, x, y, color = 'black', use_alpha = TRUE) {
  invisible(.Call(nr_point_, nr, x, y, color, use_alpha))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw lines on a native raster image
#'
#' Uses Bresenham's algorithm to draw lines. No antialiasing.
#'
#' @inheritParams nr_fill
#' @inheritParams nr_polyline
#' @param x1,y1,x2,y2 Vectors of coordinates of endpoints of line
#'
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' 
#' @examples
#' w <- 200
#' h <- 150
#' nr <- nr_new(w, h, 'grey80')
#' 
#' N <- 40
#' nr_line(
#'   nr,
#'   x1 = seq(0, 2*w, length.out =  N), y1 = 0,
#'   x2 = 0, y2 = seq(0, 2 * h, length.out = N),
#'   color = rainbow(N)
#' )
#' 
#' plot(nr)
#'
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_line <- function(nr, x1, y1, x2, y2, color = 'black', linewidth = 1, use_alpha = TRUE) {
  invisible(.Call(nr_line_, nr, x1, y1, x2, y2, color, linewidth, use_alpha))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw text on a native raster image using the built-in monospaced
#' bitmapped font.
#'
#' The only font currently available is 'spleen' - a monospace
#' bitmap font from: https://github.com/fcambus/spleen
#'
#' The 'spleen' font is licensed under BSD and the license is included in this
#' package as "LICENSE-spleen.txt". 
#' To view LICENSE: 
#' 
#' \code{cat(readLines(system.file('LICENSE-spleen.txt', package = 'nara')), 
#'  sep = "\n")}
#'
#' @inheritParams nr_fill
#' @inheritParams nr_point
#' @param str character string
#' @param x,y coordinates of lower-left corner of text
#' @param hjust,vjust Justification of text relative to its \code{(x,y)}
#'        location.  Default \code{(0.5, 0.5)} to centre the text at (x, y)
#' @param fontsize height of font in pizels.  Only valid values are 8, 12 and 16.
#'        Default: 8.
#'
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'
#' @examples
#' w <- 150
#' h <- 30
#' nr <- nr_new(w, h, 'grey80')
#' 
#' nr_text_mono(nr, x = w/2, y = h/2, str = "Hello RStats!", fontsize = 16)
#' plot(nr)
#' 
#' nr <- nr_new(w, h, 'grey80')
#' nr_text_mono(nr, x = 0, y = 0, str = "Hello RStats!", hjust = 0, vjust = 1, fontsize = 16)
#' plot(nr)
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_text_mono <- function(nr, x, y, str, color = 'black', fontsize = 8L, 
                          hjust = 0.5, vjust = 0.5, use_alpha = TRUE) {
  invisible(.Call(nr_text_mono_, nr, x, y, str, color, fontsize, hjust, vjust, use_alpha))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw rectangles on a native raster image
#'
#' @inheritParams nr_fill
#' @inheritParams nr_point
#' @inheritParams nr_blit
#' @inheritParams nr_polyline
#' @param x,y coordinates of lower left corner of rectangle.  [vector]
#' @param w,h width and height of rectangle. [vector]
#' @param color outline color. Default: NA. [vector]
#' @param fill interior fill color [vector]
#'
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' 
#' @examples
#' set.seed(1)
#' w <- 200
#' h <- 150
#' nr <- nr_new(w, h, 'grey80')
#' 
#' N <- 10
#' coords <- expand.grid(x = seq(2, w, length.out = N), 
#'                       y = seq(2, h, length.out = N))
#' 
#' cols <- sample(grDevices::heat.colors(nrow(coords)))
#' 
#' nr_rect(
#'   nr,
#'   x = coords$x,
#'   y = coords$y,
#'   w = w/N - 1,
#'   h = h/N - 1,
#'   fill = cols
#' )
#' plot(nr)
#' @family drawing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_rect <- function(nr, x, y, w, h, fill = 'black', color = NA, hjust = 0, vjust = 0, linewidth = 1, use_alpha = TRUE) {
  invisible(.Call(nr_rect_, nr,  x, y, w, h, fill, color, hjust, vjust, linewidth, use_alpha))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw circles on a native raster image
#'
#' @inheritParams nr_fill
#' @inheritParams nr_point
#' @param x,y coordinates of centre of circle.  [vector]
#' @param r radius [vector]
#' @param color outline color. Default: NA. [vector]
#' @param fill interior fill color [vector]
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'
#' @examples
#' set.seed(1)
#' w <- 200
#' h <- 150
#' nr <- nr_new(w, h, 'black')
#' 
#' N <- 10
#' coords <- expand.grid(x = seq(2, w, length.out = N), 
#'                       y = seq(2, h, length.out = N))
#' 
#' cols <- sample(grDevices::terrain.colors(nrow(coords)))
#' 
#' nr_circle(
#'   nr,
#'   x = coords$x,
#'   y = coords$y,
#'   r = w/N/1.3,
#'   fill = cols
#' )
#' plot(nr)
#' @family drawing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_circle <- function(nr, x, y, r, fill = 'black', color = NA, use_alpha = TRUE) {
  invisible(.Call(nr_circle_, nr, x, y, r, fill, color, use_alpha))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw a polyline on a native raster image
#'
#' @inheritParams nr_fill
#' @inheritParams nr_point
#' @param linewidth Line linewidth. Default: 1.  If \code{linewidth = 1} then a 
#'        naive version of Bresenham is used to draw the points.  If linewidth 
#'        is greater than 1, then the line is convert to a triangle strip and
#'        rendered as polygons.
#' @param mitre_limit Limit the size of the mitre when two lines meet at an 
#'        acute angle and linewidth is greater than 1. Default: same as line 
#'        linewidth which mostly looks OK.
#' @param close Should the polyline be closed? I.e. should a line be drawn between
#'        the last point and the first point?   Default: FALSE
#'        
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'
#' @examples
#' w <- 200
#' h <- 150
#' nr <- nr_new(w, h, 'grey80')
#' 
#' N  <- 20
#' theta <- seq(0, 2 * pi, length.out = N)
#' xs <- w/2 + 50 * cos(theta)
#' ys <- h/2 + 50 * sin(theta)
#' 
#' nr_polyline(nr, xs, ys)
#' nr_point(nr, xs, ys, col = 'red')
#' plot(nr)
#' @family drawing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_polyline <- function(nr, x, y, color = 'black', linewidth = 1, mitre_limit = linewidth, close = FALSE, use_alpha = TRUE) {
  invisible(.Call(nr_polyline_, nr, x, y, color, linewidth, mitre_limit, close, use_alpha))
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw multiple polygons on a native raster image
#'
#' @inheritParams nr_fill
#' @inheritParams nr_point
#' @inheritParams nr_polyline
#' @param fill fill color 
#' @param id integer vector used to separate coordinates into 
#'        multiple polygons. Consecutive runs of the same \code{id}
#'        value belong to the same polygon.  If NULL (the default) then 
#'        all coordinates are assumed to be vertices of a single polygon.
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'
#' @examples
#' w <- 200
#' h <- 150
#' nr <- nr_new(w, h, 'grey80')
#' 
#' N  <- 20
#' theta <- seq(0, 2 * pi, length.out = N)
#' xs <- w/2 + 50 * cos(theta)
#' ys <- h/2 + 50 * sin(theta)
#' 
#' nr_polygon(nr, xs, ys, fill = 'grey50', col = 'black')
#' nr_point(nr, xs, ys, col = 'red')
#' plot(nr)
#' @family drawing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_polygon <- function(nr, x, y, id = NULL, fill = 'black', color = NA, linewidth = 1, mitre_limit = linewidth, use_alpha = TRUE) {
  invisible(.Call(nr_polygons_multi_, nr, x, y, id = id, fill, color, linewidth, mitre_limit, use_alpha))
}

test_that("nr-draw line works", {
  
  nr <- nr_new(5, 5, 'white')
  
  nr_line(nr, 0, 0, 4, 4)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  
  expect_equal(
    nr,
    matrix(c(
      black, white, white, white, white,
      white, black, white, white, white,
      white, white, black, white, white,
      white, white, white, black, white,
      white, white, white, white, black
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
  
})



test_that("nr-draw rect works", {
  
  nr <- nr_new(5, 5, 'white')
  nr_rect(nr, 0, 0, 2, 2)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  
  expect_equal(
    nr,
    matrix(c(
      black, black, white, white, white,
      black, black, white, white, white,
      white, white, white, white, white,
      white, white, white, white, white,
      white, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
  
})


test_that("nr-draw rect works - justify 0,0", {
  
  nr <- nr_new(5, 5, 'white')
  nr_rect(nr, 2, 2, 3, 3, hjust = 0, vjust = 0)
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  
  expect_equal(
    nr,
    matrix(c(
      white, white, white, white, white,
      white, white, white, white, white,
      white, white, black, black, black,
      white, white, black, black, black,
      white, white, black, black, black
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
  
})


test_that("nr-draw rect works - justify 1,1", {
  
  nr <- nr_new(5, 5, 'white')
  nr_rect(nr, 2, 2, 3, 3, hjust = 1, vjust = 1)
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  
  expect_equal(
    nr,
    matrix(c(
      black, black, black, white, white,
      black, black, black, white, white,
      black, black, black, white, white,
      white, white, white, white, white,
      white, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
  
})


test_that("nr-draw rect works - justify 0.5, 0.5", {
  
  nr <- nr_new(5, 5, 'white')
  nr_rect(nr, 2, 2, 3, 3, hjust = 0.5, vjust = 0.5)
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  
  expect_equal(
    nr,
    matrix(c(
      white, white, white, white, white,
      white, black, black, black, white,
      white, black, black, black, white,
      white, black, black, black, white,
      white, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
  
})


test_that("nr-draw rect works - justify 0.5, 0.5 border", {
  
  nr <- nr_new(5, 5, 'white')
  nr_rect(nr, 2, 2, 3, 3, hjust = 0.5, vjust = 0.5, color = 'red')
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  red   <- colorfast::col_to_int('red')
  
  expect_equal(
    nr,
    matrix(c(
      white, white, white, white, white,
      white, red  , red  , red  , white,
      white, red  , black, red  , white,
      white, red  , red  , red  , white,
      white, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})




test_that("nr-draw circle works", {
  
  nr <- nr_new(5, 5, 'white')
  nr_circle(nr, 2, 2, 2)
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  red   <- colorfast::col_to_int('red')
  
  expect_equal(
    nr,
    matrix(c(
      white, black, black, black, white,
      black, black, black, black, black,
      black, black, black, black, black,
      black, black, black, black, black,
      white, black, black, black, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})


test_that("nr-draw circle works with border", {
  
  nr <- nr_new(5, 5, 'white')
  nr_circle(nr, 2, 2, 2, col = 'red')
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  red   <- colorfast::col_to_int('red')
  
  expect_equal(
    nr,
    matrix(c(
      white, red  , red  , red  , white,
      red  , black, black, black, red  ,
      black, black, black, black, black,
      red  , black, black, black, red  ,
      white, red  , red  , red  , white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})





test_that("nr-draw polyline works", {
  
  nr <- nr_new(5, 5, 'white')
  nr_polyline(nr, c(0, 0, 4), c(4, 0, 0))
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  red   <- colorfast::col_to_int('red')
  
  expect_equal(
    nr,
    matrix(c(
      black, black, black, black, black,
      black, white, white, white, white,
      black, white, white, white, white,
      black, white, white, white, white,
      black, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})




test_that("nr-draw polyline works with close", {
  
  nr <- nr_new(5, 5, 'white')
  nr_polyline(nr, c(0, 0, 4), c(4, 0, 0), close = TRUE)
  # plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  red   <- colorfast::col_to_int('red')
  
  expect_equal(
    nr,
    matrix(c(
      black, black, black, black, black,
      black, white, white, black, white,
      black, white, black, white, white,
      black, black, white, white, white,
      black, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})



test_that("nr-draw polygon works", {
  
  nr <- nr_new(5, 5, 'white')
  # nr_polygon(nr, c(0, 0, 4), c(4, 0, 0))
  nr_polygon(nr, c(4, 0, 0), c(0, 0, 4))
  if (interactive()) plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  red   <- colorfast::col_to_int('red')
  
  expect_equal(
    nr,
    matrix(c(
      black, black, black, black, white,
      black, black, black, black, white,
      black, black, black, white, white,
      black, black, white, white, white,
      black, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})


test_that("nr-draw polygon works with outline", {
  
  nr <- nr_new(5, 5, 'white')
  nr_polygon(nr, c(4, 0, 0), c(0, 0, 4), color = 'green')
  if (interactive()) plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('white')
  green <- colorfast::col_to_int('green')
  
  expect_equal(
    nr,
    matrix(c(
      green, green, green, green, green,
      green, black, black, green, white,
      green, black, green, white, white,
      green, green, white, white, white,
      green, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})


test_that("nr-draw text works", {
  
  nr <- nr_new(width = 5, height = 5, 'azure')
  nr_text_mono(nr, 0, 5, ".", hjust = 0, vjust = 0)
  if (interactive()) plot(nr)
  
  black <- colorfast::col_to_int('black')
  white <- colorfast::col_to_int('azure')
  green <- colorfast::col_to_int('green')
  
  expect_equal(
    nr,
    matrix(c(
      white, white, white, white, white,
      white, white, white, white, white,
      white, white, white, white, white,
      white, white, white, black, white,
      white, white, white, white, white
    ), 5, 5, byrow = T), ignore_attr = TRUE
  )
})

test_that("zero-zero at top-left - point", {
  
  nr <- nr_new(3, 2)
  nr_point(nr, 0, 0, -2L)
  expect_identical(  
    as.vector(nr), 
    c(
      -2L, -1L, -1L, 
      -1L, -1L, -1L
    )
  )
  
  
  nr <- nr_new(3, 2)
  nr_point(nr, 1, 0, -2L)
  expect_identical(  
    as.vector(nr), 
    c(
      -1L, -2L, -1L, 
      -1L, -1L, -1L
    )
  )
  
  
  nr <- nr_new(3, 2)
  nr_point(nr, 1, 1, -2L)
  expect_identical(  
    as.vector(nr), 
    c(
      -1L, -1L, -1L, 
      -1L, -2L, -1L
    )
  )
  
  
  nr <- nr_new(3, 2)
  nr_point(nr, 2, 2, -2L)
  expect_identical(  
    as.vector(nr), 
    c(
      -1L, -1L, -1L, 
      -1L, -1L, -1L
    )
  )
})




test_that("zero-zero at top-left - rect", {
  
  nr <- nr_new(4, 3)
  nr_rect(nr, 0, 0, 2, 2, fill = -2L)
  nr
  expect_identical(  
    as.vector(nr), 
    c(
      -2L, -2L, -1L, -1L, 
      -2L, -2L, -1L, -1L,
      -1L, -1L, -1L, -1L
    )
  )
  
  
  nr <- nr_new(4, 3)
  nr_rect(nr, 1, 1, 2, 2, fill = -2L)
  # nr
  expect_identical(  
    as.vector(nr), 
    c(
      -1L, -1L, -1L, -1L, 
      -1L, -2L, -2L, -1L,
      -1L, -2L, -2L, -1L
    )
  )
  
  
  nr <- nr_new(4, 3)
  nr_rect(nr, 2, 2, 2, 2, fill = -2L)
  # nr
  expect_identical(  
    as.vector(nr), 
    c(
      -1L, -1L, -1L, -1L,
      -1L, -1L, -1L, -1L,
      -1L, -1L, -2L, -2L
    )
  )
  
  if (FALSE) {
    nr <- nr_new(4, 3) |>
      nr_rect(2, 2, 2, 2, fill = 'black') |>
      plot(T)
    nr
  }
  
  
})

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Draw multiple triangles from mesh data
#' 
#' Mesh data is a similar format to the \code{mesh3d} class of data 
#' defined in the \code{rgl} package.
#' 
#' @inheritParams nr_fill
#' @inheritParams nr_rect
#' @param vertices Wide matrix of vertex coordinates where the first two rows are
#'        x and y coordinates.  Extra coordinates are ignored.
#' @param indices 3xN wide integer matrix of index information. Each column 
#'        holds the 3 indices which indicate the column index of the triangle vertices defined in the
#'        \code{vertices} argument.
#' @param coords Wide numeric matrix of direct coordinate data for each triangle where
#'        the first two rows are x and y coordinates (extra rows are ignored)
#'        Each group of 3 columns defines the (x,y) coordinates for one
#'        triangle.
#' @param color color specification. Single color or one color per tri
#' @param tris Which triangles should be drawn?  Valid options: 'all' (default), 
#'        'ccw', 'cw'. The options 'ccw' and 'cw' limit plotting to those triangles
#'        where the order of the vertices are defined in a counter-clockwise
#'        or clockwise manner respectively.  Note: when considering orientation
#'        or triangles remember that the y-axis is defined vertically \emph{down}
#'        the screen. 
#' 
#' @examples
#' #' Using direct coordinates
#' set.seed(1)
#' w <- 100
#' h <-  80
#' nr <- nr_new(w, h)
#' 
#' n_tri <- 10
#' xs <- runif(n_tri * 3, 0, w - 1)
#' ys <- runif(n_tri * 3, 0, h - 1)
#' coords <- rbind(xs, ys)
#' cols <- rainbow(n_tri)
#' 
#' nr_tri_coords(nr, coords, cols, tris = 'all')
#' plot(nr)
#' 
#' # Using a standard mesh structure
#' # i.e. a matrix of vertices, and a matrix of indices 
#' nr <- nr_new(w, h)
#' 
#' # Matrix of coordinates
#' xs <- rep(seq(0, w - 1, length.out = 10), 10)
#' ys <- rep(seq(0, h - 1, length.out = 10), each = 10)
#' vertices <- rbind(xs, ys)
#' nr_point(nr, xs, ys)
#' 
#' plot(nr)
#' 
#' # Matrix of indices indicating which vertices make up each triangle
#' indices <- matrix(sample(length(xs), 3 * n_tri), nrow = 3) 
#' 
#' nr_tri_mesh(nr, vertices, indices, cols, tris = 'all')
#' plot(nr)
#' 
#' @return Invisibly return the supplied native raster image which was been modified in-place
#' @family drawing functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_tri_mesh <- function(nr, vertices, indices, color, tris = 'all', use_alpha = TRUE) {
  invisible(
    .Call(nr_tri_mesh_, nr, vertices, indices, color, tris, use_alpha)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' @rdname nr_tri_mesh
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_tri_coords <- function(nr, coords, color, tris = 'all', use_alpha = TRUE) {
  invisible(
    .Call(nr_tri_coords_, nr, coords, color, tris, use_alpha)
  )
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Move image colors to gray
#' 
#' @inheritParams nr_fill
#' @param factor desaturation factor. Default: 1 (fully desaturate)
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'         
#' @examples
#' plot(deer[[1]])
#' nr <- deer[[1]] |> nr_copy()
#' 
#' nr_desaturate(nr)
#' plot(nr)
#' @family color manipulation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_desaturate <- function(nr, factor = 1) {
  invisible(
    .Call(nr_desaturate_, nr, factor)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Dither to binary image
#' 
#' @inheritParams nr_fill
#' @param value Threshold value. Default: 0.5  (valid range [0, 1])
#' @param algo Dithering algorithm. 'fs' (floyd-steinberg) or 'atkinson'. 
#'        Default: 'fs'
#'        
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'         
#' @examples
#' nr <- nr_copy(deer[[1]])
#' plot(nr)
#' 
#' nr_dither(nr, 0.99)
#' plot(nr)
#' @family color manipulation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_dither <- function(nr, value = 0.5, algo = 'fs') {
  invisible(
    .Call(nr_dither_, nr, value, algo)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Threshold to binary image
#' 
#' @inheritParams nr_fill
#' @param value Threshold value. Default: 0.5  (valid range [0, 1])
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#'         
#' @examples
#' nr <- nr_copy(deer[[1]])
#' plot(nr)
#' 
#' nr_threshold(nr, 0.9)
#' plot(nr)
#' @family color manipulation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_threshold <- function(nr, value = 0.5) {
  invisible(
    .Call(nr_threshold_, nr, value)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Replace colors in a native raster image
#' 
#' @inheritParams nr_fill
#' @param old Vector of old colors
#' @param new Vector of replacement colors
#' 
#' @return Invisibly return the supplied native raster image which was been
#'         modified in-place
#' 
#' @examples
#' nr <- nr_new(10, 10, 'hotpink')
#' nr_color_replace(nr, 'hotpink', 'grey80')
#' plot(nr)
#' @family color manipulation functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_color_replace <- function(nr, old, new) {
  invisible(
    .Call(nr_color_replace_, nr, old, new)
  )
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Copy a native raster image into another at an arbitrary location, scale and rotation
#'
#' Copy a whole image, or subset of an image, into the destination image and 
#' configure the angle and size of the copied image.
#'
#' This operation is vectorised such that a single \code{src} image can be 
#' pasted multiple times into the \code{dst} image with varying location, 
#' angle and scale.
#'
#' @inheritParams nr_point
#' @param dst,src source and destination native raster images
#' @param x,y Location in \code{dst} to place the \code{src} image. These values must
#'        be vectors of the same length.  If the length is greater than 1, then
#'        the \code{src} will be pasted into \code{dst} at multiple locations.
#' @param xsrc,ysrc Upper-left coordinates of the bounding box within the \code{src}
#'        image to copy.  Default: (0, 0)
#' @param w,h Width and height of the bounding box within the \code{src} to copy.
#'        If size is negative, then the actual width/height of
#'        the src is used. Default: (-1, -1)
#' @param hjust,vjust specify horizontal and vertical justification of the 
#'        handle on the \code{src} image to be placed at \code{(x, y)} within 
#'        the \code{dst} image.
#'        Default (0, 0) is to use the top-left corner of the image as the 
#'        handle.
#'        Use (0.5, 0.5) to centre the \code{src} image at the \code{dst} 
#'        location \code{(x, y)}
#' @param angle Rotation angle (clockwise) in radians. Default: 0.
#'        If this is a vector, then the \code{src} will be pasted into the
#'        \code{dst} at multiple angles.
#' @param scale Zoom factor. Default: 1.  IF this is a vector, then the \code{src}
#'        will be pasted into \code{dst} at multiple scales.
#'
#' @return Invisibly return the supplied \code{dst} native raster image which was been
#'         modified in-place
#' 
#' @examples
#' nr <- nr_new(50, 50, 'grey80')
#' nr_blit(dst = nr, src = deer[[1]], x = 25, y = 25)
#' plot(nr)
#' 
#' nr <- nr_new(300, 200, 'grey80')
#' sq <- nr_new(20, 20, 'darkblue')
#' nr_blit(nr, src = sq, x = 100, y = 100, angle = pi/3, scale = 5)
#' plot(nr)
#'
#' nr <- nr_new(800, 600, 'grey80')
#' sq <- fastpng::read_png(system.file("image/deer-1.png", package="nara"), type = 'nativeraster')
#' nr_blit(nr, src = sq, x = 300, y = 240, angle = pi/2, scale = 1)
#' plot(nr)
#' @family blitting functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_blit <- function(dst, src, x, y, 
                    xsrc = 0L, ysrc = 0L, 
                    w = -1L, h = -1L,
                    hjust = 0.5, vjust = 0.5, 
                    angle = 0, scale = 1,
                    use_alpha = TRUE) {
  invisible(
    .Call(nr_blit_, 
          dst, x, y, 
          src, xsrc, ysrc, 
          w, h,
          hjust, vjust,
          angle, scale, 
          use_alpha)
  )
}



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' Multiple blit operations in a single call
#' 
#' @param dst destination native raster
#' @param src list of native raster images.
#' @param config data.frame of configuration information for each blit.
#'    Columns in this data.frame match the arguments to \code{nr_blit()}.
#'    Requires columns:
#'    \describe{
#'      \item{\code{idx}}{The index of the image in the \code{src} list i.e. \code{src[[idx]]}}
#'      \item{\code{x,y}}{location in \code{dst} to place \code{src[[idx]]}}
#'    } 
#'    Optional columns which take a default value if not present:
#'    \describe{
#'      \item{\code{xsrc,ysrc}}{start coordinates within src. Default: (0, 0)}
#'      \item{\code{w,h}}{size within \code{src}. Default: use whole width/height}
#'      \item{\code{hjust,vjust}}{Default: (0.5, 0.5)}
#'      \item{\code{angle}}{Default: 0}
#'      \item{\code{scale}}{Zoom factor. Default: 1}
#'      \item{\code{use_alpha}}{Default: true}
#'      \item{\code{render}}{Should this given operation be rendered? Default: true}
#'    }
#'    
#' @return Invisibly return the supplied \code{dst} native raster image which was been
#'         modified in-place
#'           
#' @examples
#' nr <- nr_new(90, 90, 'grey60')
#' 
#' config <- data.frame(
#'   idx = c(1, 2, 3, 4),
#'   x = c(10, 10, 40, 40) + 15,
#'   y = c(10, 40, 40, 10) + 15,
#'   xsrc = 0L,
#'   ysrc = 0L,
#'   w = -1L,
#'   h = -1L,
#'   hjust = 0.5,
#'   vjust = 0.5,
#'   angle = c(0, 0, 0, pi/4),
#'   scale = c(0.5, 1, 1, 1),
#'   use_alpha = TRUE,
#'   render = TRUE
#' )
#' 
#' nr_blit_multi(dst = nr, src = deer, config = config)
#' plot(nr)
#' @family blitting functions
#' @export
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr_blit_multi <- function(dst, src, config) {
  invisible(
    .Call(nr_blit_multi_, dst, src, config)
  )
}

---
title: "C API"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{C API}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(nara)
library(callme)
```


C function declaration for drawing a line on a native raster

`void nr_line(uint32_t *nr, int nr_width, int nr_height, int x1, int y1, int x2, int y2, uint32_t color, double linewidth);`

* `nr` is a `uint32_t` pointer to the native raster data
* `nr_width` and `nr_height` are the number of columns and rows of the raster
* `x1, y1, x2, y2` are the integer coordinates of the endpoints of the line
* `color` is a packaged integer color of the form `0xAABBGGRRu`.  See also the `{colorfast}` package.


```{r}
code <- r"(
#include <stdint.h>
#include "nara.h"

SEXP my_func(SEXP nr_) {

  int nr_width = Rf_ncols(nr_);
  int nr_height = Rf_nrows(nr_);
  uint32_t *nr = (uint32_t *)INTEGER(nr_);
  uint32_t col = 0xFF0000FFu; // 0xAABBGGRR Red
  bool use_alpha = true;

  //      nr, nr_width, nr_height, x1, y1,  x2,  y2, color, linewidth, use_alpha
  nr_line(nr, nr_width, nr_height,  0,  0, 200, 150, col  , 1,         use_alpha);

  return nr_;
}
)"

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Find the location of nara.h and include its directory in the search path
# using C Pre-Processor flags (PKG_CPPFLAGS)
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nara_h <- system.file("include/private/nara.h", package = "nara", mustWork = TRUE)
cpp_include = paste0("-I", dirname(nara_h))
callme::compile(code, PKG_CPPFLAGS = cpp_include)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Create a native raster and call my function
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr <- nr_new(400, 300, 'grey70')
nr <- my_func(nr)
plot(nr)
```

---
title: "Using Masks"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Masks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, echo=FALSE}
set.seed(1)
library(nara)
```


`{nara}` supports masked rendering of most drawing operations.

`blit_mask_begin()` and `blit_mask_end()` are used to delineate the use
of a mask on an image.  Within these bounds, any operation will only 
affect the pixels in the image corresponding to where the mask is not transparent.

Note: masks to not need to be rectangular or contiguous.  Anywhere the
mask is opaque is part of the valid masked area for drawing.

```{r}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Setup a drawing canvas and an image to use as a mask
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
w <- 300
h <- 300
canvas <- nr_new(w, h, fill = 'lightblue')
mask   <- nr_new(w, h, fill = 'transparent')

# Set the top-left area of the mask to be the active area by making it 
# not transparent
nr_rect(mask, x=0, y=0, w=w/2, h=h/2)
plot(mask)

# Attach the mask to the canvas
nr_mask_begin(canvas, mask)
{
  # Draw random circles
  xs <- runif(50, 0, w)
  ys <- runif(50, 0, h)
  nr_circle(canvas, xs, ys, 20, rainbow(50))
}
nr_mask_end(canvas) # Finalise the masked operation


plot(canvas)
```

---
title: "Triangles"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Triangles}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(nara)
```


`{nara}` has three ways to draw triangles:

* `nr_polygon()`
* `nr_tri_mesh()`
* `nr_tri_coords()`

For single triangles, `nr_polygon()` is sufficient.  For large numbers of
polygons, the `nr_tri_*` functions will be much much faster.

This vignette demonstrates the drawing of two triangles using each of these 
functions.


## Demo setup

Define 5 points on the image which will be used as the vertices of two triangles


```{r}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Define image size 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
w <- h <- 300
th <- 16 # text height

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Define 5 points within the canvas
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ax <- w/2     ; ay <- 0 + th
bx <- w/2     ; by <- h/2     
cx <-     th/2; cy <- h/2     
dx <- w/2     ; dy <- h - th/2           
ex <- w - th/2; ey <- h/2

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Label the points in an image
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
nr <- nr_new(w, h)
nr_text_mono(nr, ax, ay, 'A', fontsize = th)
nr_text_mono(nr, bx, by, 'B', fontsize = th)
nr_text_mono(nr, cx, cy, 'C', fontsize = th)
nr_text_mono(nr, dx, dy, 'D', fontsize = th)
nr_text_mono(nr, ex, ey, 'E', fontsize = th)

plot(nr)
```



# Render triangles with `nr_polygon()`

Render two triangles by using two calls to `nr_polygon()`.

```{r echo = FALSE}
nr <- nr_new(w, h)

nr_polygon(nr, c(ax, bx, cx), c(ay, by, cy), fill = "blue")
nr_polygon(nr, c(bx, dx, ex), c(by, dy, cy), fill = "red")

plot(nr)
```


# Render triangles with `nr_tri_mesh()`

`nr_tri_mesh()` expects the triangles to be defined in two parts:

1. A matrix of vertex coordinates 
    *  `x` values in the first row, `y` values 
        in the second row.  (Further rows are ignored)
2. An integer matrix of indices.
    * The first 3 values in each column define the columns in the `vertices`
      matrix which define the triangle.
      
This mesh data definition is a similar format to the `mesh3d` class of data 
defined in the `{rgl}` package.

```{r}
# define all the vertex coordinates
vertices <- rbind(
  c(ax, bx, cx, dx, ex),
  c(ay, by, cy, dy, ey)
)

# Define the indices.  Each column in this matrix represents 1 triangle.
# The three numbers reprsent the column index in the `vertices` data
indices <- rbind(
  # 1  2
  c(1, 2),
  c(2, 4),
  c(3, 5)
)


nr <- nr_new(w, h)
nr_tri_mesh(nr, vertices, indices, c('blue', 'red'))

plot(nr)
```


# Render triangles with `nr_tri_coords()`

`nr_tri_coords()` requires just the raw coordinates of all vertices in a single
matrix.  The first row has the `x` coordinates, the second row has the `y`
coordinates.

Each group of 3 columns defines a triangle

```{r}
coords <- rbind(
  # <<<  1 >>>    <<<  2 >>>>
  c(ax, bx, cx,   bx, dx, ex),
  c(ay, by, cy,   by, dy, ey)
)

nr <- nr_new(w, h)
nr_tri_coords(nr, coords, color = c('blue', 'red'))

plot(nr)
```

---
title: "Animation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Animation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, echo = FALSE}
library(nara)
```


Animating with `{nara}` is simply a matter of generating and displaying 
multiple images - hopefully quickly enough to look smooth.

There are two approaches:

1. Render all the images into a list, and then save as a video file e.g. `.gif`, `.mp4`
2. Display the images to a fast graphics device


## Render to a video file

To render to a video file, `{nara}` use `nrs_to_gif()` and `nrs_to_mp4()`.
These functions are simple wrappers around the 
[`{magick}`](https://cran.r-project.org/package=magick) package.

The code below:

* sets up `N` circles with positions and velocities
* for each frame
    * create a new image
    * advance the circle positions
    * draw circles
    * add image to the list of all images
* saves the list of images to a `.gif`

```{r eval=FALSE}
library(nara)
w <- 320
h <- 200

set.seed(1)
N <- 20
xs   <- runif(N, 0, w)
ys   <- runif(N, 0, h)
rs   <- runif(N, 2, 6)  
vx   <- rnorm(N, sd = 3)
cols <- sample(heat.colors(N))

# Storage for animation frames
anim <- list()

for (i in seq(30)) {
  # create image
  nr <- nr_new(w, h, fill = 'grey90')
  
  # update the x position and wraparound
  xs <- xs + vx
  xs <- ifelse(xs < 0, xs + w, xs)
  xs <- ifelse(xs > w, xs - w, xs)
  
  # draw circles
  nr_circle(nr, xs, ys, rs, cols)
  
  # store animatino frame
  anim[[i]] <- nr  
}

nrs_to_gif(anim, "working/demo.gif")
# nrs_to_mp4(anim, "working/demo.mp4")
```


## Render to a fast graphics device

If your graphics device updates fast enough, then you can show images on this
device quickly and have the appearance on smooth animation.

A major problem for most platforms: **Rstudio and Positron graphics devices 
are not fast enough to do this**

Instead, if you are wanting to use a built-in graphics device, the
`x11()` device is the preferred graphics output for animation.

`x11()` device is supported on macos, linux and windows (where it a lightweight 
wrapper around the standard `windows()` device).

A good way to setup this device for fast animation is to:

1. Call `x11()`
2. Set the type to double buffered Cairo rendering `type = 'dbcairo'`
3. Disable the *display list* as there's no need to record graphics operations
4. Use `dev.hold()` and `dev.flush()` calls to synchronise the drawing command
   with the graphics display update (this can prevent tearing).
5. A small pause may be needed in the loop, otherwise things might go *too* fast!

See code below for an example:

```{r eval=FALSE}
x11(..., type = 'dbcairo', antialias = 'none')
dev.control(displaylist = 'inhibit')

for (i in seq(nframes)) {
  nr <- generate_image_somehow()
  dev.hold()
  plot(nr)
  dev.flush()
  Sys.sleep(0.03) # Sleep maybe?
}
```


Note: Particular platforms may have differing parameters to give good results for
realtime animation - you might need to experiment to get the best out of your 
platform.

---
title: "Technical Notes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Technical Notes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(nara)
```



### What is a native raster image and why is it fast?


A native raster image is a built-in datatype in R.

It is an integer matrix where each integer represents the RGBA color at 
a single pixel.  The 32-bit integer at each location is interpreted within
R to be four color channels (RGBA) represented by 8 bits each.

This way of encoding color information is closer to the internal representation
used by graphics devices, and therefore can be faster to render and manipulate.

Native rasters do **not** use pre-multiplied alpha.

### In-place operation

`{nara}` tries to minimise
R function calls and memory allocations in order to be fast.

When updating native raster image with this package, changes are done
*in place* on the current image i.e. a new image is **not** created.

### Anti-aliasing/Interpolation

No anti-aliasing is done by the draw methods in this package.

No interpolation is done - `x` and `y` values for drawing coordinates are
always rounded to integers.

### Dimension ordering

All arguments specifying dimensions are in the order **horizontal** then **vertical** i.e.

* x, y
* width, height
* hjust, vjust

### Coordinate System

The coordinate system for `nara` native raster image has the origin
at the **top left corner** with coordinates `(0, 0)`.

This is equivalent to `{grid}` graphics using `native` units.

It is also how `{magick}` represents image coordinates, as well as the majority
of C graphics libraries.

---
title: "Blitting"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Blitting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(1)
```

```{r setup, echo = FALSE}
library(nara)
```

[Blitting](https://en.wikipedia.org/wiki/Bit_blit) is a term used in computer
graphics to describe the copying of whole or part of one image into another.

This copying is optinised for speed and forms a significant part of the 
underlying mechanics for 2D games and interactive applications.  For example:

* putting a players character on the scrolling background in a game
* Assembling a background image from a small library of tiled images
* Placing pre-rendered pieces on a game board


`{nara}` includes two blitting functions:

* `nr_blit()`
    * Blit a single source image at one (or multiple) locations in the destination
      image.
* `nr_blit_multi()`
    * Define a complex sequence of blitting operations using a data.frame
    
# `nr_blit()`

Use `nr_blit()` when you want to use a single source image only
    
### Blit a single image

```{r}
library(nara)

# Create a canvas to draw on
w <- 300
h <- 300
nr <- nr_new(w, h, 'grey90')

# Place a deer on the canvas
nr_blit(nr, deer[[1]], w/2, h/2)

plot(nr)
```    
    
    
### Blit a single image to multiple locations


```{r}
# Create a canvas to draw on
nr <- nr_new(w, h, 'grey90')

# Place a deer on the canvas at many locations
xs <- runif(20, 0, w)
ys <- runif(20, 0, h)
nr_blit(nr, deer[[1]], xs, ys)

plot(nr)
```
    
    
### Blit a single image to multiple locations with varying angles and scales


```{r}
# Create a canvas to draw on
nr <- nr_new(w, h, 'grey90')

# Place a deer on the canvas at many locations
xs <- runif(20, 0, w)
ys <- runif(20, 0, h)
angles <- runif(20, 0, 2*pi)
scales <- runif(20, 0.5, 2)
nr_blit(nr, deer[[1]], xs, ys, angle = angles, scale = scales)

plot(nr)
```



# `nr_blit_multi()`

If you have multiple source images to blit, there are two possible approaches:

* Loop over the images and blit them one-at-a-time with `nr_blit()`
* Set-up a configuration data.frame and blit them all-at-once with `nr_blit_multi()`

For most simple tasks, `nr_blit()` should be sufficient.

For larger structures, it may be useful to store blitting parameters in a data.frame
for organisation purposes. 

The `{nara}` packages includes a `tileset` (a list of nativerasters which 
can *tile* together to form an image) and a `tileset_config` giving one 
possible arrangement of tiles.

Combining `tileset` with `tileset_config` in `nr_blit_multi()` creates a 
complex image with a single function call.


```{r}
head(tileset_config)

w <- 70 * 11
h <- 70 * 8
nr <- nr_new(w, h, fill = 'lightblue')
nr_blit_multi(nr, tileset, config = tileset_config)

plot(nr)
```





### Blit with a mask

`{nara}` supports masked rendering of most drawing operations.

`blit_mask_begin()` and `blit_mask_end()` are used to delineate the use
of a mask on an image.

```{r}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Setup a drawing canvas and an image to use as a mask
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
w <- 300
h <- 300
canvas <- nr_new(w, h, fill = 'lightblue')
mask   <- nr_new(w, h, fill = 'transparent')

# Set the top-left area of the mask to be the active area by making it 
# not transparent
nr_rect(mask, x=0, y=0, w=w/2, h=h/2)
plot(mask)

# Attach the mask to the canvas
nr_mask_begin(canvas, mask)
{
  # Draw random deer on the image - 
  xs <- runif(50, 0, w)
  ys <- runif(50, 0, h)
  nr_blit(canvas, deer[[1]], xs, ys)
}
nr_mask_end(canvas) # Finalise the masked operation


plot(canvas)
```

---
title: "Convert images to/from native raster image format"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Convert images to/from native raster image format}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(grid)
library(nara)
```


## Create a native raster image

```{r}
nr <- nr_new(10, 10, fill = 'white')
class(nr)
```



## Save/Load native raster images to/from PNG and JPEG

The `{fastpng}`, `{png}` and `{jpeg}` packages allow images to be loaded directly 
as native raster images.


```{r}
png_file <- system.file("image", "deer-1.png", package="nara")
jpg_file <- system.file("image", "deer-1.jpg", package="nara")
```



```{r}
nr <- fastpng::read_png(png_file, type = 'nativeraster')
class(nr)
plot(nr)
```


```{r}
nr <- png::readPNG(png_file, native = TRUE)
class(nr)
plot(nr)
```

```{r}
nr <- jpeg::readJPEG(jpg_file, native = TRUE)
class(nr)
plot(nr)
```


These packages will also automatically detect and save native raster images

```{r eval=FALSE}
fastpng::write_png(nr, "test.png")
jpeg::writeJPEG(nr, "test.jpg")
png::writePNG(nr, "test.png")
```

## Convert a regular R `raster` image to/from native raster image format

```{r}
ras <- matrix(c('brown', 'darkblue', 'darkblue', 'brown'), 2, 2)
ras
class(ras)
grid.raster(ras, interpolate = FALSE)
```


```{r}
nr <- nara::raster_to_nr(ras)
nr
class(nr)
plot(nr)
```

## Convert a regular R `array` image to/from native raster image format

```{r}
arr <- array(c(
  1, 1, 0,
  0, 0, 0,
  0, 0, 0, 
  1, 1, 0
), dim = c(2, 2, 3))

arr
class(arr)
grid.raster(arr, interpolate = FALSE)
```

```{r}
nr <- nara::array_to_nr(arr)
nr
class(nr)
plot(nr)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-magick.R
\name{magick_to_nr}
\alias{magick_to_nr}
\alias{nr_to_magick}
\title{Convert a 'magick' image to native raster image}
\usage{
magick_to_nr(im, dst = NULL)

nr_to_magick(nr)
}
\arguments{
\item{im}{image from the \code{magick} package}

\item{dst}{destination native raster image. If NULL (the default) a 
new native raster image will be created  If a native raster image 
is supplied here, it must have the exact dimensions to match the matrix}

\item{nr}{native raster image}
}
\value{
New native raster image
}
\description{
Convert a 'magick' image to native raster image
}
\examples{
\dontshow{if (interactive() && requireNamespace('magick', quietly = TRUE)) withAutoprint(\{ # examplesIf}
im <- magick::logo
nr <- magick_to_nr(im)
plot(nr)
\dontshow{\}) # examplesIf}
}
\seealso{
Other conversion functions: 
\code{\link{matrix_to_nr}()},
\code{\link{nr_to_raster}()},
\code{\link{nrs_to_gif}()},
\code{\link{nrs_to_mp4}()}

Other conversion functions: 
\code{\link{matrix_to_nr}()},
\code{\link{nr_to_raster}()},
\code{\link{nrs_to_gif}()},
\code{\link{nrs_to_mp4}()}
}
\concept{conversion functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nara-package.R
\docType{package}
\name{nara-package}
\alias{nara}
\alias{nara-package}
\title{nara: Native Raster Image Tools}
\description{
Native rasters are a core R image format which use a compact color representation. This color representation closely aligns with graphics device internals meaning that these images can be rendered quickly. This package provides functions to quickly create, manipulate and composite native rasters.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/coolbutuseless/nara}
  \item \url{https://coolbutuseless.github.io/package/nara/index.html}
  \item Report bugs at \url{https://github.com/coolbutuseless/nara/issues}
}

}
\author{
\strong{Maintainer}: Mike Cheng \email{mikefc@coolbutuseless.com} [copyright holder]

Other contributors:
\itemize{
  \item Julien Vernay (Author of public domain code for thick polylines) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{is_nativeraster}
\alias{is_nativeraster}
\title{Check if object is a native raster image}
\usage{
is_nativeraster(x)
}
\arguments{
\item{x}{object to check}
}
\value{
logical. TRUE if object is a native raster image
}
\description{
Check if object is a native raster image
}
\examples{
is_nativeraster(mtcars)

nr <- nr_new(100, 100)
is_nativeraster(nr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-color.R
\name{nr_color_replace}
\alias{nr_color_replace}
\title{Replace colors in a native raster image}
\usage{
nr_color_replace(nr, old, new)
}
\arguments{
\item{nr}{native raster image}

\item{old}{Vector of old colors}

\item{new}{Vector of replacement colors}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Replace colors in a native raster image
}
\examples{
nr <- nr_new(10, 10, 'hotpink')
nr_color_replace(nr, 'hotpink', 'grey80')
plot(nr)
}
\seealso{
Other color manipulation functions: 
\code{\link{nr_desaturate}()},
\code{\link{nr_dither}()},
\code{\link{nr_threshold}()}
}
\concept{color manipulation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-scale.R
\name{nr_resize}
\alias{nr_resize}
\title{Resize a native raster by specifying the output dimensions}
\usage{
nr_resize(nr, width, height, algo = "nn")
}
\arguments{
\item{nr}{native raster image}

\item{width, height}{dimensions for output image}

\item{algo}{'nn' for nearest neighbor (the default), or 'bilinear' for 
bilinear interpolation.}
}
\value{
New native raster image
}
\description{
Resize a native raster by specifying the output dimensions
}
\examples{
stretched <- deer[[1]] |> 
    nr_copy() |> 
    nr_resize(100, 40, algo = 'nn')
plot(stretched)
}
\seealso{
Other resizing functions: 
\code{\link{nr_scale}()}
}
\concept{resizing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-sets.R
\docType{data}
\name{tileset}
\alias{tileset}
\title{A list of nativeraster tiles used for constructing a side-scrolling game.}
\format{
An object of class \code{list} of length 44.
}
\usage{
tileset
}
\description{
The tiles in this list are of a common base size and can be conbined to 
construct various scenes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-sets.R
\docType{data}
\name{tileset_config}
\alias{tileset_config}
\title{A data.frame of configuration information which can be used with 
\code{nr_blit_multi()} and the \code{tileset} data.}
\format{
An object of class \code{spec_tbl_df} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 51 rows and 6 columns.
}
\usage{
tileset_config
}
\description{
Use this configuration wtih \code{nr_blit_multi()}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{print.nativeRaster}
\alias{print.nativeRaster}
\title{Print method}
\usage{
\method{print}{nativeRaster}(x, ...)
}
\arguments{
\item{x}{native raster image}

\item{...}{ignored}
}
\value{
None
}
\description{
Print method
}
\examples{
nr <- nr_new(500, 400)
print(nr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-blit.R
\name{nr_blit}
\alias{nr_blit}
\title{Copy a native raster image into another at an arbitrary location, scale and rotation}
\usage{
nr_blit(
  dst,
  src,
  x,
  y,
  xsrc = 0L,
  ysrc = 0L,
  w = -1L,
  h = -1L,
  hjust = 0.5,
  vjust = 0.5,
  angle = 0,
  scale = 1,
  use_alpha = TRUE
)
}
\arguments{
\item{dst, src}{source and destination native raster images}

\item{x, y}{Location in \code{dst} to place the \code{src} image. These values must
be vectors of the same length.  If the length is greater than 1, then
the \code{src} will be pasted into \code{dst} at multiple locations.}

\item{xsrc, ysrc}{Upper-left coordinates of the bounding box within the \code{src}
image to copy.  Default: (0, 0)}

\item{w, h}{Width and height of the bounding box within the \code{src} to copy.
If size is negative, then the actual width/height of
the src is used. Default: (-1, -1)}

\item{hjust, vjust}{specify horizontal and vertical justification of the 
handle on the \code{src} image to be placed at \code{(x, y)} within 
the \code{dst} image.
Default (0, 0) is to use the top-left corner of the image as the 
handle.
Use (0.5, 0.5) to centre the \code{src} image at the \code{dst} 
location \code{(x, y)}}

\item{angle}{Rotation angle (clockwise) in radians. Default: 0.
If this is a vector, then the \code{src} will be pasted into the
\code{dst} at multiple angles.}

\item{scale}{Zoom factor. Default: 1.  IF this is a vector, then the \code{src}
will be pasted into \code{dst} at multiple scales.}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied \code{dst} native raster image which was been
        modified in-place
}
\description{
Copy a whole image, or subset of an image, into the destination image and 
configure the angle and size of the copied image.
}
\details{
This operation is vectorised such that a single \code{src} image can be 
pasted multiple times into the \code{dst} image with varying location, 
angle and scale.
}
\examples{
nr <- nr_new(50, 50, 'grey80')
nr_blit(dst = nr, src = deer[[1]], x = 25, y = 25)
plot(nr)

nr <- nr_new(300, 200, 'grey80')
sq <- nr_new(20, 20, 'darkblue')
nr_blit(nr, src = sq, x = 100, y = 100, angle = pi/3, scale = 5)
plot(nr)

nr <- nr_new(800, 600, 'grey80')
sq <- fastpng::read_png(system.file("image/deer-1.png", package="nara"), type = 'nativeraster')
nr_blit(nr, src = sq, x = 300, y = 240, angle = pi/2, scale = 1)
plot(nr)
}
\seealso{
Other blitting functions: 
\code{\link{nr_blit_multi}()}
}
\concept{blitting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-draw.R
\name{nr_point}
\alias{nr_point}
\title{Draw points on a native raster image}
\usage{
nr_point(nr, x, y, color = "black", use_alpha = TRUE)
}
\arguments{
\item{nr}{native raster image}

\item{x, y}{Vectors of point coordinates}

\item{color}{Vector of colors}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Draw points on a native raster image
}
\examples{
w <- 200
h <- 150
nr <- nr_new(w, h, 'grey80')

coords <- expand.grid(x = seq(w) - 1, y = seq(h) - 1)
cols   <- sample(rainbow(nrow(coords)))
nr_point(nr, x = coords$x, y = coords$y, color = cols) 
plot(nr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-draw.R
\name{nr_line}
\alias{nr_line}
\title{Draw lines on a native raster image}
\usage{
nr_line(nr, x1, y1, x2, y2, color = "black", linewidth = 1, use_alpha = TRUE)
}
\arguments{
\item{nr}{native raster image}

\item{x1, y1, x2, y2}{Vectors of coordinates of endpoints of line}

\item{color}{Color as a character string. Either a standard R color
(e.g. 'blue', 'white')
or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
or \code{#rgb}}

\item{linewidth}{Line linewidth. Default: 1.  If \code{linewidth = 1} then a 
naive version of Bresenham is used to draw the points.  If linewidth 
is greater than 1, then the line is convert to a triangle strip and
rendered as polygons.}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Uses Bresenham's algorithm to draw lines. No antialiasing.
}
\examples{
w <- 200
h <- 150
nr <- nr_new(w, h, 'grey80')

N <- 40
nr_line(
  nr,
  x1 = seq(0, 2*w, length.out =  N), y1 = 0,
  x2 = 0, y2 = seq(0, 2 * h, length.out = N),
  color = rainbow(N)
)

plot(nr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-scale.R
\name{nr_scale}
\alias{nr_scale}
\title{Resize a native raster image using a scale factor}
\usage{
nr_scale(nr, scale, algo = "nn")
}
\arguments{
\item{nr}{native raster image}

\item{scale}{scale factor}

\item{algo}{'nn' for nearest neighbor (the default), or 'bilinear' for 
bilinear interpolation.}
}
\value{
New native raster image
}
\description{
Resize a native raster image using a scale factor
}
\examples{
big <- deer[[1]] |> nr_scale(2)
plot(big)
}
\seealso{
Other resizing functions: 
\code{\link{nr_resize}()}
}
\concept{resizing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-tri.R
\name{nr_tri_mesh}
\alias{nr_tri_mesh}
\alias{nr_tri_coords}
\title{Draw multiple triangles from mesh data}
\usage{
nr_tri_mesh(nr, vertices, indices, color, tris = "all", use_alpha = TRUE)

nr_tri_coords(nr, coords, color, tris = "all", use_alpha = TRUE)
}
\arguments{
\item{nr}{native raster image}

\item{vertices}{Wide matrix of vertex coordinates where the first two rows are
x and y coordinates.  Extra coordinates are ignored.}

\item{indices}{3xN wide integer matrix of index information. Each column 
holds the 3 indices which indicate the column index of the triangle vertices defined in the
\code{vertices} argument.}

\item{color}{color specification. Single color or one color per tri}

\item{tris}{Which triangles should be drawn?  Valid options: 'all' (default), 
'ccw', 'cw'. The options 'ccw' and 'cw' limit plotting to those triangles
where the order of the vertices are defined in a counter-clockwise
or clockwise manner respectively.  Note: when considering orientation
or triangles remember that the y-axis is defined vertically \emph{down}
the screen.}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}

\item{coords}{Wide numeric matrix of direct coordinate data for each triangle where
the first two rows are x and y coordinates (extra rows are ignored)
Each group of 3 columns defines the (x,y) coordinates for one
triangle.}
}
\value{
Invisibly return the supplied native raster image which was been modified in-place
}
\description{
Mesh data is a similar format to the \code{mesh3d} class of data 
defined in the \code{rgl} package.
}
\examples{
#' Using direct coordinates
set.seed(1)
w <- 100
h <-  80
nr <- nr_new(w, h)

n_tri <- 10
xs <- runif(n_tri * 3, 0, w - 1)
ys <- runif(n_tri * 3, 0, h - 1)
coords <- rbind(xs, ys)
cols <- rainbow(n_tri)

nr_tri_coords(nr, coords, cols, tris = 'all')
plot(nr)

# Using a standard mesh structure
# i.e. a matrix of vertices, and a matrix of indices 
nr <- nr_new(w, h)

# Matrix of coordinates
xs <- rep(seq(0, w - 1, length.out = 10), 10)
ys <- rep(seq(0, h - 1, length.out = 10), each = 10)
vertices <- rbind(xs, ys)
nr_point(nr, xs, ys)

plot(nr)

# Matrix of indices indicating which vertices make up each triangle
indices <- matrix(sample(length(xs), 3 * n_tri), nrow = 3) 

nr_tri_mesh(nr, vertices, indices, cols, tris = 'all')
plot(nr)

}
\seealso{
Other drawing functions: 
\code{\link{nr_circle}()},
\code{\link{nr_polygon}()},
\code{\link{nr_polyline}()},
\code{\link{nr_rect}()}
}
\concept{drawing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-draw.R
\name{nr_rect}
\alias{nr_rect}
\title{Draw rectangles on a native raster image}
\usage{
nr_rect(
  nr,
  x,
  y,
  w,
  h,
  fill = "black",
  color = NA,
  hjust = 0,
  vjust = 0,
  linewidth = 1,
  use_alpha = TRUE
)
}
\arguments{
\item{nr}{native raster image}

\item{x, y}{coordinates of lower left corner of rectangle.  [vector]}

\item{w, h}{width and height of rectangle. [vector]}

\item{fill}{interior fill color [vector]}

\item{color}{outline color. Default: NA. [vector]}

\item{hjust, vjust}{specify horizontal and vertical justification of the 
handle on the \code{src} image to be placed at \code{(x, y)} within 
the \code{dst} image.
Default (0, 0) is to use the top-left corner of the image as the 
handle.
Use (0.5, 0.5) to centre the \code{src} image at the \code{dst} 
location \code{(x, y)}}

\item{linewidth}{Line linewidth. Default: 1.  If \code{linewidth = 1} then a 
naive version of Bresenham is used to draw the points.  If linewidth 
is greater than 1, then the line is convert to a triangle strip and
rendered as polygons.}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Draw rectangles on a native raster image
}
\examples{
set.seed(1)
w <- 200
h <- 150
nr <- nr_new(w, h, 'grey80')

N <- 10
coords <- expand.grid(x = seq(2, w, length.out = N), 
                      y = seq(2, h, length.out = N))

cols <- sample(grDevices::heat.colors(nrow(coords)))

nr_rect(
  nr,
  x = coords$x,
  y = coords$y,
  w = w/N - 1,
  h = h/N - 1,
  fill = cols
)
plot(nr)
}
\seealso{
Other drawing functions: 
\code{\link{nr_circle}()},
\code{\link{nr_polygon}()},
\code{\link{nr_polyline}()},
\code{\link{nr_tri_mesh}()}
}
\concept{drawing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_copy}
\alias{nr_copy}
\title{Create a new native raster image and copy the dimensions and
contents from an existing image}
\usage{
nr_copy(nr)
}
\arguments{
\item{nr}{native raster image}
}
\value{
New native raster image
}
\description{
Create a new native raster image and copy the dimensions and
contents from an existing image
}
\examples{
nr1 <- nr_new(200, 200, 'hotpink')
nr2 <- nr_copy(nr1)
plot(nr2)
}
\seealso{
Other image creation functions: 
\code{\link{nr_new}()}
}
\concept{image creation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-color.R
\name{nr_threshold}
\alias{nr_threshold}
\title{Threshold to binary image}
\usage{
nr_threshold(nr, value = 0.5)
}
\arguments{
\item{nr}{native raster image}

\item{value}{Threshold value. Default: 0.5  (valid range [0, 1])}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Threshold to binary image
}
\examples{
nr <- nr_copy(deer[[1]])
plot(nr)

nr_threshold(nr, 0.9)
plot(nr)
}
\seealso{
Other color manipulation functions: 
\code{\link{nr_color_replace}()},
\code{\link{nr_desaturate}()},
\code{\link{nr_dither}()}
}
\concept{color manipulation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_flipv}
\alias{nr_flipv}
\title{Flip a native raster image vertically}
\usage{
nr_flipv(nr)
}
\arguments{
\item{nr}{native raster image}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Flip a native raster image vertically
}
\examples{
nr <- deer[[1]] |> nr_copy()
plot(nr)
nr_flipv(nr)
plot(nr)
}
\seealso{
Other transformation functions: 
\code{\link{nr_fliph}()},
\code{\link{nr_rotate}()},
\code{\link{nr_transpose}()}
}
\concept{transformation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{nr_to_raster}
\alias{nr_to_raster}
\alias{raster_to_nr}
\alias{nr_to_array}
\alias{array_to_nr}
\title{Convert native raster images to/from other R objects}
\usage{
nr_to_raster(nr)

raster_to_nr(ras, dst = NULL)

nr_to_array(nr)

array_to_nr(arr, dst = NULL)
}
\arguments{
\item{nr}{native raster image}

\item{ras}{standard R raster i.e. a character matrix of hex color values}

\item{dst}{destination native raster image. If NULL (the default) a new}

\item{arr}{3d numeric array representing R,G,B,A values with dimensions [nrow, ncol, 4] or
[nrow, ncol, 3]. Each value is in range [0,1].}
}
\value{
raster, array or native raster image
}
\description{
Convert native raster images to/from other R objects
}
\examples{
nr <- nr_new(12, 8, 'hotpink')
nr_to_raster(nr)

nr_to_array(nr)
}
\seealso{
Other conversion functions: 
\code{\link{magick_to_nr}()},
\code{\link{matrix_to_nr}()},
\code{\link{nrs_to_gif}()},
\code{\link{nrs_to_mp4}()}
}
\concept{conversion functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-magick.R
\name{nrs_to_gif}
\alias{nrs_to_gif}
\title{Convert a list of native rasters to an animated gif}
\usage{
nrs_to_gif(nr_list, filename, verbose = FALSE, framerate = 30, ...)
}
\arguments{
\item{nr_list}{list of native raster image}

\item{filename}{filename of gif}

\item{verbose}{logical. default FALSE}

\item{framerate}{frames per second}

\item{...}{other arguments passed to \code{magick::image_write_gif()}}
}
\value{
None.
}
\description{
Convert a list of native rasters to an animated gif
}
\examples{
\dontshow{if (interactive() && requireNamespace('magick', quietly = TRUE)) withAutoprint(\{ # examplesIf}
im <- magick::logo
nr1 <- magick_to_nr(im)
nr2 <- nr_copy(nr1)
nrs_to_gif(list(nr1, nr2), "nothing.gif")
\dontshow{\}) # examplesIf}
}
\seealso{
Other conversion functions: 
\code{\link{magick_to_nr}()},
\code{\link{matrix_to_nr}()},
\code{\link{nr_to_raster}()},
\code{\link{nrs_to_mp4}()}
}
\concept{conversion functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-blit.R
\name{nr_blit_multi}
\alias{nr_blit_multi}
\title{Multiple blit operations in a single call}
\usage{
nr_blit_multi(dst, src, config)
}
\arguments{
\item{dst}{destination native raster}

\item{src}{list of native raster images.}

\item{config}{data.frame of configuration information for each blit.
Columns in this data.frame match the arguments to \code{nr_blit()}.
Requires columns:
\describe{
  \item{\code{idx}}{The index of the image in the \code{src} list i.e. \code{src[[idx]]}}
  \item{\code{x,y}}{location in \code{dst} to place \code{src[[idx]]}}
} 
Optional columns which take a default value if not present:
\describe{
  \item{\code{xsrc,ysrc}}{start coordinates within src. Default: (0, 0)}
  \item{\code{w,h}}{size within \code{src}. Default: use whole width/height}
  \item{\code{hjust,vjust}}{Default: (0.5, 0.5)}
  \item{\code{angle}}{Default: 0}
  \item{\code{scale}}{Zoom factor. Default: 1}
  \item{\code{use_alpha}}{Default: true}
  \item{\code{render}}{Should this given operation be rendered? Default: true}
}}
}
\value{
Invisibly return the supplied \code{dst} native raster image which was been
        modified in-place
}
\description{
Multiple blit operations in a single call
}
\examples{
nr <- nr_new(90, 90, 'grey60')

config <- data.frame(
  idx = c(1, 2, 3, 4),
  x = c(10, 10, 40, 40) + 15,
  y = c(10, 40, 40, 10) + 15,
  xsrc = 0L,
  ysrc = 0L,
  w = -1L,
  h = -1L,
  hjust = 0.5,
  vjust = 0.5,
  angle = c(0, 0, 0, pi/4),
  scale = c(0.5, 1, 1, 1),
  use_alpha = TRUE,
  render = TRUE
)

nr_blit_multi(dst = nr, src = deer, config = config)
plot(nr)
}
\seealso{
Other blitting functions: 
\code{\link{nr_blit}()}
}
\concept{blitting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-color.R
\name{nr_desaturate}
\alias{nr_desaturate}
\title{Move image colors to gray}
\usage{
nr_desaturate(nr, factor = 1)
}
\arguments{
\item{nr}{native raster image}

\item{factor}{desaturation factor. Default: 1 (fully desaturate)}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Move image colors to gray
}
\examples{
plot(deer[[1]])
nr <- deer[[1]] |> nr_copy()

nr_desaturate(nr)
plot(nr)
}
\seealso{
Other color manipulation functions: 
\code{\link{nr_color_replace}()},
\code{\link{nr_dither}()},
\code{\link{nr_threshold}()}
}
\concept{color manipulation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-draw.R
\name{nr_text_mono}
\alias{nr_text_mono}
\title{Draw text on a native raster image using the built-in monospaced
bitmapped font.}
\usage{
nr_text_mono(
  nr,
  x,
  y,
  str,
  color = "black",
  fontsize = 8L,
  hjust = 0.5,
  vjust = 0.5,
  use_alpha = TRUE
)
}
\arguments{
\item{nr}{native raster image}

\item{x, y}{coordinates of lower-left corner of text}

\item{str}{character string}

\item{color}{Color as a character string. Either a standard R color
(e.g. 'blue', 'white')
or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
or \code{#rgb}}

\item{fontsize}{height of font in pizels.  Only valid values are 8, 12 and 16.
Default: 8.}

\item{hjust, vjust}{Justification of text relative to its \code{(x,y)}
location.  Default \code{(0.5, 0.5)} to centre the text at (x, y)}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
The only font currently available is 'spleen' - a monospace
bitmap font from: https://github.com/fcambus/spleen
}
\details{
The 'spleen' font is licensed under BSD and the license is included in this
package as "LICENSE-spleen.txt". 
To view LICENSE: 

\code{cat(readLines(system.file('LICENSE-spleen.txt', package = 'nara')), 
 sep = "\n")}
}
\examples{
w <- 150
h <- 30
nr <- nr_new(w, h, 'grey80')

nr_text_mono(nr, x = w/2, y = h/2, str = "Hello RStats!", fontsize = 16)
plot(nr)

nr <- nr_new(w, h, 'grey80')
nr_text_mono(nr, x = 0, y = 0, str = "Hello RStats!", hjust = 0, vjust = 1, fontsize = 16)
plot(nr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-draw.R
\name{nr_polyline}
\alias{nr_polyline}
\title{Draw a polyline on a native raster image}
\usage{
nr_polyline(
  nr,
  x,
  y,
  color = "black",
  linewidth = 1,
  mitre_limit = linewidth,
  close = FALSE,
  use_alpha = TRUE
)
}
\arguments{
\item{nr}{native raster image}

\item{x, y}{Vectors of point coordinates}

\item{color}{Color as a character string. Either a standard R color
(e.g. 'blue', 'white')
or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
or \code{#rgb}}

\item{linewidth}{Line linewidth. Default: 1.  If \code{linewidth = 1} then a 
naive version of Bresenham is used to draw the points.  If linewidth 
is greater than 1, then the line is convert to a triangle strip and
rendered as polygons.}

\item{mitre_limit}{Limit the size of the mitre when two lines meet at an 
acute angle and linewidth is greater than 1. Default: same as line 
linewidth which mostly looks OK.}

\item{close}{Should the polyline be closed? I.e. should a line be drawn between
the last point and the first point?   Default: FALSE}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Draw a polyline on a native raster image
}
\examples{
w <- 200
h <- 150
nr <- nr_new(w, h, 'grey80')

N  <- 20
theta <- seq(0, 2 * pi, length.out = N)
xs <- w/2 + 50 * cos(theta)
ys <- h/2 + 50 * sin(theta)

nr_polyline(nr, xs, ys)
nr_point(nr, xs, ys, col = 'red')
plot(nr)
}
\seealso{
Other drawing functions: 
\code{\link{nr_circle}()},
\code{\link{nr_polygon}()},
\code{\link{nr_rect}()},
\code{\link{nr_tri_mesh}()}
}
\concept{drawing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_copy_into}
\alias{nr_copy_into}
\title{Copy the contents of one native raster image into another.}
\usage{
nr_copy_into(dst, src)
}
\arguments{
\item{src, dst}{Source and destination native raster images}
}
\value{
Invisibly return the supplied 'dst' native raster image which was been
        modified in-place
}
\description{
The source and destination native raster images must have the same dimensions.
}
\details{
If the native raster images are of different sizes or alpha blending is
required, use the \code{\link{nr_blit}()} function.
}
\examples{
nr1 <- nr_new(200, 100, 'hotpink')
nr2 <- nr_new(200, 100, 'green')
nr_copy_into(nr1, nr2)
plot(nr1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-sets.R
\docType{data}
\name{deer}
\alias{deer}
\title{Deer images - a list of native raster images}
\format{
An object of class \code{list} of length 15.
}
\usage{
deer
}
\description{
The 'deer' sprites are by Calciumtrice and licensed under CC BY 3.0.  
See 'inst/LICENSE-deer.txt' for more details. To view license information:
}
\details{
\code{cat(readLines(system.file('LICENSE-deer.txt', package = 'nara')), sep = "\n")}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-mask.R
\name{nr_mask_begin}
\alias{nr_mask_begin}
\alias{nr_mask_end}
\title{Start/end region of masked drawing}
\usage{
nr_mask_begin(nr, mask)

nr_mask_end(nr)
}
\arguments{
\item{nr}{native raster image}

\item{mask}{native raster image to use as mask. Must be the same size as
\code{nr}.}
}
\value{
Invisibly return the original native raster image which has been 
        modified in-place
}
\description{
These functions are used to globally set a mask which determines where
pixels are affected by drawing functions.
}
\details{
Drawing operations between
\code{nr_mask_begin()} and \code{nr_mask_end()} calls will only affect pixels
where the \code{mask} is not transparent.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
mask <- deer[[1]] |> nr_resize(600, 400)
plot(mask)
nr <- nr_new_from(mask)

nr_mask_begin(nr, mask = mask)
nr_circle(nr, 300, 200, 200, fill = 'blue', color = 'black')
nr_circle(nr, 350, 200, 100, fill = 'darkgreen', color = 'black')
nr_mask_end(nr)

plot(nr)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_crop}
\alias{nr_crop}
\alias{nr_crop2}
\title{Crop a section out of a native raster image into a new image}
\usage{
nr_crop(nr, x, y, w, h)

nr_crop2(nr, loc)
}
\arguments{
\item{nr}{native raster image}

\item{x, y, w, h}{dimensions of cropped section}

\item{loc}{dimensions of cropped section. A vector of 4 values 
i.e. \code{c(x, y, w, h)}}
}
\value{
New native raster image
}
\description{
Crop a section out of a native raster image into a new image
}
\examples{
nr <- deer[[1]]
dim(nr)
plot(nr)

nr2 <- nr_crop(nr, 16, 0, 16, 16)
dim(nr2)
plot(nr2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_new}
\alias{nr_new}
\alias{nr_new_from}
\title{Create a native raster image}
\usage{
nr_new(width, height, fill = "white")

nr_new_from(nr, fill = "white")
}
\arguments{
\item{width, height}{Image dimensions in pixels}

\item{fill}{Background fill color as a character string. Either a standard R color
(e.g. 'blue', 'white')
or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
or \code{#rgb}}

\item{nr}{native raster image to use as the template for the size of the 
new image in \code{nr_new_from()}}
}
\value{
native raster image
}
\description{
A native raster image in R looks like an integer matrix, but is interpreted
differently by graphics devices:
}
\details{
\itemize{
  \item{The data should be treated as RGBA pixels in row-major ordering}
  \item{Each 32-bit integer should be interpreted as 4-bytes - one for each
        of the R, G, B and A color channels}
}
}
\examples{
nr <- nr_new(400, 300, 'hotpink')
plot(nr)

}
\seealso{
Other image creation functions: 
\code{\link{nr_copy}()}
}
\concept{image creation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_fill}
\alias{nr_fill}
\title{Fill a native raster image with the given color}
\usage{
nr_fill(nr, color)
}
\arguments{
\item{nr}{native raster image}

\item{color}{Color as a character string. Either a standard R color
(e.g. 'blue', 'white')
or a hex color of the form \code{#rrggbbaa}, \code{#rrggbb}, \code{#rgba}
or \code{#rgb}}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Fill a native raster image with the given color
}
\examples{
nr <- nr_new(400, 300, 'hotpink')
nr_fill(nr, 'blue')
plot(nr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-draw.R
\name{nr_circle}
\alias{nr_circle}
\title{Draw circles on a native raster image}
\usage{
nr_circle(nr, x, y, r, fill = "black", color = NA, use_alpha = TRUE)
}
\arguments{
\item{nr}{native raster image}

\item{x, y}{coordinates of centre of circle.  [vector]}

\item{r}{radius [vector]}

\item{fill}{interior fill color [vector]}

\item{color}{outline color. Default: NA. [vector]}

\item{use_alpha}{Use alpha channel when drawing? Logical. Default: TRUE}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Draw circles on a native raster image
}
\examples{
set.seed(1)
w <- 200
h <- 150
nr <- nr_new(w, h, 'black')

N <- 10
coords <- expand.grid(x = seq(2, w, length.out = N), 
                      y = seq(2, h, length.out = N))

cols <- sample(grDevices::terrain.colors(nrow(coords)))

nr_circle(
  nr,
  x = coords$x,
  y = coords$y,
  r = w/N/1.3,
  fill = cols
)
plot(nr)
}
\seealso{
Other drawing functions: 
\code{\link{nr_polygon}()},
\code{\link{nr_polyline}()},
\code{\link{nr_rect}()},
\code{\link{nr_tri_mesh}()}
}
\concept{drawing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{plot.nativeRaster}
\alias{plot.nativeRaster}
\title{Plot a native raster image (after first clearning the device)}
\usage{
\method{plot}{nativeRaster}(x, y, ...)
}
\arguments{
\item{x}{native raster image}

\item{y}{ignored}

\item{...}{other arguments passed to \code{grid::grid.raster()}}
}
\value{
Invisibly return the supplied native raster image
}
\description{
Plot a native raster image (after first clearning the device)
}
\examples{
nr <- nr_new(200, 100, 'hotpink')
plot(nr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-color.R
\name{nr_dither}
\alias{nr_dither}
\title{Dither to binary image}
\usage{
nr_dither(nr, value = 0.5, algo = "fs")
}
\arguments{
\item{nr}{native raster image}

\item{value}{Threshold value. Default: 0.5  (valid range [0, 1])}

\item{algo}{Dithering algorithm. 'fs' (floyd-steinberg) or 'atkinson'. 
Default: 'fs'}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Dither to binary image
}
\examples{
nr <- nr_copy(deer[[1]])
plot(nr)

nr_dither(nr, 0.99)
plot(nr)
}
\seealso{
Other color manipulation functions: 
\code{\link{nr_color_replace}()},
\code{\link{nr_desaturate}()},
\code{\link{nr_threshold}()}
}
\concept{color manipulation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nr-core.R
\name{nr_fliph}
\alias{nr_fliph}
\title{Flip a native raster image horizontally}
\usage{
nr_fliph(nr)
}
\arguments{
\item{nr}{native raster image}
}
\value{
Invisibly return the supplied native raster image which was been
        modified in-place
}
\description{
Flip a native raster image horizontally
}
\examples{
nr <- nr_new(400, 200, 'white')
nr_rect(nr, 0, 0, 30, 15)
plot(nr)
nr_fliph(nr)
plot(nr)
}
\seealso{
Other transformation functions: 
\code{\link{nr_flipv}()},
\code{\link{nr_rotate}()},
\code{\link{nr_transpose}()}
}
\concept{transformation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{matrix_to_nr}
\alias{matrix_to_nr}
\title{Convert a numeric matrix to native raster image}
\usage{
matrix_to_nr(mat, palette, fill = "transparent", min = 0, max = 1, dst = NULL)
}
\arguments{
\item{mat}{integer matrix}

\item{palette}{vector of colors.  For an integer matrix, this palette must 
contain at least as many colors as the maximum integer value in \code{mat}. 
For a numeric matrix, any length palette is allowed, and the nearest
corresponding color is chosen (no interpolation is done).}

\item{fill}{Color to be used for values < 1 when input is an integer matrix.
Default: 'transparent'.}

\item{min, max}{assumed range for the numeric data.  values from the palette
will be interpolated using this range as the extents.  An error
will occur if a value lies outside this range. Default: (0, 1)}

\item{dst}{destination native raster image. If NULL (the default) a 
new native raster image will be created  If a native raster image 
is supplied here, it must have the exact dimensions to match the matrix}
}
\value{
native raster image
}
\description{
Convert a numeric matrix to native raster image
}
\examples{
# integer matrix
m <- matrix(1:12, 3, 4)
m
palette <- rainbow(12)
nr <- matrix_to_nr(m, palette) 
plot(nr)


# numeric matrix
m <- matrix(runif(20 * 30), 20, 30)
palette <- c('red', 'blue', 'black', 'green', 'blue', 'yellow')
nr <- matrix_to_nr(m, palette)
plot(nr)
}
\seealso{
Other conversion functions: 
\code{\link{magick_to_nr}()},
\code{\link{nr_to_raster}()},
\code{\link{nrs_to_gif}()},
\code{\link{nrs_to_mp4}()}
}
\concept{conversion functions}
